@@ -60,8 +60,9 @@ void RSession::login()
     m_rServer = new org::kde::Cantor::R(QString::fromLatin1("org.kde.Cantor.R-%1").arg(m_process->pid()),  QLatin1String("/"), QDBusConnection::sessionBus(), this);
 
     connect(m_rServer, SIGNAL(statusChanged(int)), this, SLOT(serverChangedStatus(int)));
-    connect(m_rServer, SIGNAL(symbolList(const QStringList&, const QStringList&, const QStringList&)),this,SLOT(receiveSymbols(const QStringList&, const QStringList&, const QStringList&)));
+    connect(m_rServer, SIGNAL(symbolList(QStringList,QStringList,QStringList)),this,SLOT(receiveSymbols(QStringList,QStringList,QStringList)));
 
+    changeStatus(Session::Done);
     emit loginDone();
     qDebug()<<"login done";
 }
@@ -70,43 +71,53 @@ void RSession::logout()
 {
     qDebug()<<"logout";
     m_process->terminate();
+
+    changeStatus(Status::Disable);
 }
 
 void RSession::interrupt()
 {
-    const int pid = m_process->pid();
-    qDebug()<<"interrupt" << pid;
-    if (pid)
+    if(!expressionQueue().isEmpty())
     {
+        qDebug()<<"interrupting " << expressionQueue().first()->command();
+        if(m_process->state() != QProcess::NotRunning)
+        {
 #ifndef Q_OS_WIN
-        kill(pid, SIGINT);
+            const int pid=m_process->pid();
+            kill(pid, SIGINT);
 #else
-      //TODO: interrupt the process on windows
+            ; //TODO: interrupt the process on windows
 #endif
+        }
+        expressionQueue().first()->interrupt();
+        expressionQueue().removeFirst();
+        foreach (Cantor::Expression* expression, expressionQueue())
+            expression->setStatus(Cantor::Expression::Done);
+        expressionQueue().clear();
+
+        qDebug()<<"done interrupting";
     }
-    m_expressionQueue.removeFirst();
+
     changeStatus(Cantor::Session::Done);
 }
 
-Cantor::Expression* RSession::evaluateExpression(const QString& cmd, Cantor::Expression::FinishingBehavior behave)
+Cantor::Expression* RSession::evaluateExpression(const QString& cmd, Cantor::Expression::FinishingBehavior behave, bool internal)
 {
     qDebug()<<"evaluating: "<<cmd;
-    RExpression* expr=new RExpression(this);
+    RExpression* expr=new RExpression(this, internal);
     expr->setFinishingBehavior(behave);
     expr->setCommand(cmd);
 
     expr->evaluate();
 
-    changeStatus(Cantor::Session::Running);
-
     return expr;
 }
 
 Cantor::CompletionObject* RSession::completionFor(const QString& command, int index)
 {
     RCompletionObject *cmp=new RCompletionObject(command, index, this);
-    connect(m_rServer,SIGNAL(completionFinished(const QString&,const QStringList&)),cmp,SLOT(receiveCompletions(const QString&,const QStringList&)));
-    connect(cmp,SIGNAL(requestCompletion(const QString&)),m_rServer,SLOT(completeCommand(const QString&)));
+    connect(m_rServer,SIGNAL(completionFinished(QString,QStringList)),cmp,SLOT(receiveCompletions(QString,QStringList)));
+    connect(cmp,SIGNAL(requestCompletion(QString)),m_rServer,SLOT(completeCommand(QString)));
     return cmp;
 }
 
@@ -115,6 +126,7 @@ QSyntaxHighlighter* RSession::syntaxHighlighter(QObject* parent)
     RHighlighter *h=new RHighlighter(parent);
     connect(h,SIGNAL(syntaxRegExps(QVector<QRegExp>&,QVector<QRegExp>&)),this,SLOT(fillSyntaxRegExps(QVector<QRegExp>&,QVector<QRegExp>&)));
     connect(this,SIGNAL(symbolsChanged()),h,SLOT(refreshSyntaxRegExps()));
+    connect(this,SIGNAL(syntaxRegExpsFilled()), h, SLOT(updateHighlighting()));
     return h;
 }
 
@@ -130,6 +142,8 @@ void RSession::fillSyntaxRegExps(QVector<QRegExp>& v, QVector<QRegExp>& f)
     foreach (const QString s, m_functions)
         if (!s.contains(QRegExp(QLatin1String("[^A-Za-z0-9_.]"))))
             f.append(QRegExp(QLatin1String("\\b")+s+QLatin1String("\\b")));
+
+    emit syntaxRegExpsFilled();
 }
 
 void RSession::receiveSymbols(const QStringList& vars, const QStringList& values, const QStringList & funcs)
@@ -144,50 +158,43 @@ void RSession::receiveSymbols(const QStringList& vars, const QStringList& values
     emit symbolsChanged();
 }
 
-void RSession::queueExpression(RExpression* expr)
-{
-    m_expressionQueue.append(expr);
-
-    if(status()==Cantor::Session::Done)
-        QTimer::singleShot(0, this, SLOT(runNextExpression()));
-}
-
-
 void RSession::serverChangedStatus(int status)
 {
     qDebug()<<"changed status to "<<status;
     if(status==0)
     {
-        if(!m_expressionQueue.isEmpty())
+        if(!expressionQueue().isEmpty())
         {
-            RExpression* expr=m_expressionQueue.takeFirst();
+            RExpression* expr = static_cast<RExpression*>(expressionQueue().takeFirst());
             qDebug()<<"done running "<<expr<<" "<<expr->command();
         }
 
-        if(m_expressionQueue.isEmpty())
+        if(expressionQueue().isEmpty())
             changeStatus(Cantor::Session::Done);
         else
-            runNextExpression();
+            runFirstExpression();
     }
     else
         changeStatus(Cantor::Session::Running);
 }
 
-void RSession::runNextExpression()
+void RSession::runFirstExpression()
 {
-    if (m_expressionQueue.isEmpty())
-	return;
-    disconnect(m_rServer,  SIGNAL(expressionFinished(int, const QString&)),  0,  0);
-    disconnect(m_rServer, SIGNAL(inputRequested(const QString&)), 0, 0);
-    disconnect(m_rServer, SIGNAL(showFilesNeeded(const QStringList&)), 0, 0);
-    qDebug()<<"size: "<<m_expressionQueue.size();
-    RExpression* expr=m_expressionQueue.first();
+    if (expressionQueue().isEmpty())
+        return;
+
+    disconnect(m_rServer,  SIGNAL(expressionFinished(int,QString)),  nullptr,  nullptr);
+    disconnect(m_rServer, SIGNAL(inputRequested(QString)), nullptr, nullptr);
+    disconnect(m_rServer, SIGNAL(showFilesNeeded(QStringList)), nullptr, nullptr);
+    qDebug()<<"size: "<<expressionQueue().size();
+    RExpression* expr = static_cast<RExpression*>(expressionQueue().first());
     qDebug()<<"running expression: "<<expr->command();
 
-    connect(m_rServer, SIGNAL(expressionFinished(int,  const QString &)), expr, SLOT(finished(int, const QString&)));
-    connect(m_rServer, SIGNAL(inputRequested(const QString&)), expr, SIGNAL(needsAdditionalInformation(const QString&)));
-    connect(m_rServer, SIGNAL(showFilesNeeded(const QStringList&)), expr, SLOT(showFilesAsResult(const QStringList&)));
+    connect(m_rServer, SIGNAL(expressionFinished(int,QString)), expr, SLOT(finished(int,QString)));
+    connect(m_rServer, SIGNAL(inputRequested(QString)), expr, SIGNAL(needsAdditionalInformation(QString)));
+    connect(m_rServer, SIGNAL(showFilesNeeded(QStringList)), expr, SLOT(showFilesAsResult(QStringList)));
 
+    expr->setStatus(Cantor::Expression::Computing);
     m_rServer->runCommand(expr->command());
 }
 
@@ -202,4 +209,4 @@ void RSession::sendInputToServer(const QString& input)
 QAbstractItemModel* RSession::variableModel()
 {
     return m_variableModel;
-}
\ No newline at end of file
+}
