@@ -44,6 +44,7 @@
 //maybe this is caused by some behaviour if the Prompt is split into multiple "readStdout" calls
 //the Expressions are encapsulated in () to allow capturing for the text
 const QRegExp MaximaSession::MaximaOutputPrompt=QRegExp(QLatin1String("(\\(\\s*%\\s*o\\s*[0-9\\s]*\\))")); //Text, maxima outputs, before any output
+const QRegExp MaximaSession::MaximaInputPrompt = QRegExp(QLatin1String("(\\(\\s*%\\s*i\\s*[0-9\\s]*\\))"));
 
 
 MaximaSession::MaximaSession( Cantor::Backend* backend ) : Session(backend),
@@ -72,18 +73,18 @@ void MaximaSession::login()
     m_process->waitForReadyRead();
     qDebug()<<m_process->readAllStandardOutput();
 
-    connect(m_process, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(restartMaxima()));
+    connect(m_process, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(restartMaxima()));
     connect(m_process, SIGNAL(readyReadStandardOutput()), this, SLOT(readStdOut()));
     connect(m_process, SIGNAL(readyReadStandardError()), this, SLOT(readStdErr()));
     connect(m_process, SIGNAL(error(QProcess::ProcessError)), this, SLOT(reportProcessError(QProcess::ProcessError)));
 
-    //TODO
-//     if(!MaximaSettings::self()->autorunScripts().isEmpty()){
-//         QString autorunScripts = MaximaSettings::self()->autorunScripts().join(QLatin1String("\n"));
-//         evaluateExpression(autorunScripts, MaximaExpression::DeleteOnFinish);
-// //         runFirstExpression();
-//     }
+    if(!MaximaSettings::self()->autorunScripts().isEmpty()){
+        QString autorunScripts = MaximaSettings::self()->autorunScripts().join(QLatin1String(";"));
+        autorunScripts.append(QLatin1String(";kill(labels)")); // Reset labels after running autorun scripts
+        evaluateExpression(autorunScripts, MaximaExpression::DeleteOnFinish, true);
+    }
 
+    changeStatus(Session::Done);
     emit loginDone();
     qDebug()<<"login done";
 }
@@ -95,7 +96,7 @@ void MaximaSession::logout()
     if(!m_process)
         return;
 
-    disconnect(m_process, 0, this, 0);
+    disconnect(m_process, nullptr, this, nullptr);
 
 //     if(status()==Cantor::Session::Running)
         //TODO: terminate the running expressions first
@@ -111,17 +112,20 @@ void MaximaSession::logout()
         qDebug()<<"maxima still running, process kill enforced";
     }
 
-    m_expressionQueue.clear();
+    expressionQueue().clear();
     delete m_process;
     m_process = nullptr;
+    m_variableModel->clear();
+
+    changeStatus(Status::Disable);
 
     qDebug()<<"logout done";
 }
 
-Cantor::Expression* MaximaSession::evaluateExpression(const QString& cmd, Cantor::Expression::FinishingBehavior behave)
+Cantor::Expression* MaximaSession::evaluateExpression(const QString& cmd, Cantor::Expression::FinishingBehavior behave, bool internal)
 {
     qDebug() << "evaluating: " << cmd;
-    MaximaExpression* expr = new MaximaExpression(this);
+    MaximaExpression* expr = new MaximaExpression(this, internal);
     expr->setFinishingBehavior(behave);
     expr->setCommand(cmd);
     expr->evaluate();
@@ -129,18 +133,6 @@ Cantor::Expression* MaximaSession::evaluateExpression(const QString& cmd, Cantor
     return expr;
 }
 
-void MaximaSession::appendExpressionToQueue(MaximaExpression* expr)
-{
-    m_expressionQueue.append(expr);
-
-    qDebug()<<"queue: "<<m_expressionQueue.size();
-    if(m_expressionQueue.size()==1)
-    {
-        changeStatus(Cantor::Session::Running);
-        runFirstExpression();
-    }
-}
-
 void MaximaSession::readStdErr()
 {
    qDebug()<<"reading stdErr";
@@ -148,9 +140,9 @@ void MaximaSession::readStdErr()
        return;
    QString out=QLatin1String(m_process->readAllStandardError());
 
-   if(m_expressionQueue.size()>0)
+   if(expressionQueue().size()>0)
    {
-       MaximaExpression* expr=m_expressionQueue.first();
+       MaximaExpression* expr = static_cast<MaximaExpression*>(expressionQueue().first());
        expr->parseError(out);
    }
 }
@@ -164,7 +156,7 @@ void MaximaSession::readStdOut()
     if ( !out.contains(QLatin1String("</cantor-prompt>")) )
         return;
 
-    if(m_expressionQueue.isEmpty())
+    if(expressionQueue().isEmpty())
     {
         //queue is empty, interrupt was called, nothing to do here
         qDebug()<<m_cache;
@@ -172,26 +164,15 @@ void MaximaSession::readStdOut()
         return;
     }
 
-    MaximaExpression* expr = m_expressionQueue.first();
+    MaximaExpression* expr = static_cast<MaximaExpression*>(expressionQueue().first());
     if (!expr)
         return; //should never happen
 
-    qDebug()<<"start parsing " << "  " << m_cache;
-    if(expr->parseOutput(m_cache))
-        qDebug()<<"parsing successful";
-    else
-        qDebug() << "failed to parse";
-
+    qDebug()<<"output: " << m_cache;
+    expr->parseOutput(m_cache);
     m_cache.clear();
 }
 
-void MaximaSession::killLabels()
-{
-    Cantor::Expression* e=evaluateExpression(QLatin1String("kill(labels);"), Cantor::Expression::DeleteOnFinish);
-    e->setInternal(true);
-    //TODO: what for? connect(e, SIGNAL(statusChanged(Cantor::Expression::Status)), this, SIGNAL(ready()));
-}
-
 void MaximaSession::reportProcessError(QProcess::ProcessError e)
 {
     qDebug()<<"process error"<<e;
@@ -204,19 +185,17 @@ void MaximaSession::reportProcessError(QProcess::ProcessError e)
 
 void MaximaSession::currentExpressionChangedStatus(Cantor::Expression::Status status)
 {
-    MaximaExpression* expression=m_expressionQueue.first();
-    qDebug() << expression << status;
+    Cantor::Expression* expression = expressionQueue().first();
+    qDebug() << "expression status changed: command = " << expression->command() << ", status = " << status;
 
     if(status!=Cantor::Expression::Computing) //The session is ready for the next command
     {
-        qDebug()<<"expression finished";
+        qDebug()<<"################################## EXPRESSION END ###############################################";
         disconnect(expression, SIGNAL(statusChanged(Cantor::Expression::Status)),
                    this, SLOT(currentExpressionChangedStatus(Cantor::Expression::Status)));
 
-        qDebug()<<"running next command";
-
-        m_expressionQueue.removeFirst();
-        if(m_expressionQueue.isEmpty())
+        expressionQueue().removeFirst();
+        if(expressionQueue().isEmpty())
         {
             //if we are done with all the commands in the queue,
             //use the opportunity to update the variablemodel (if the last command wasn't already an update, as infinite loops aren't fun)
@@ -245,12 +224,13 @@ void MaximaSession::runFirstExpression()
     if (!m_process)
         return;
 
-    if(!m_expressionQueue.isEmpty())
+    if(!expressionQueue().isEmpty())
     {
-        MaximaExpression* expr=m_expressionQueue.first();
+        MaximaExpression* expr = static_cast<MaximaExpression*>(expressionQueue().first());
         QString command=expr->internalCommand();
         connect(expr, SIGNAL(statusChanged(Cantor::Expression::Status)), this, SLOT(currentExpressionChangedStatus(Cantor::Expression::Status)));
 
+        expr->setStatus(Cantor::Expression::Computing);
         if(command.isEmpty())
         {
             qDebug()<<"empty command";
@@ -266,36 +246,33 @@ void MaximaSession::runFirstExpression()
 
 void MaximaSession::interrupt()
 {
-    if(!m_expressionQueue.isEmpty())
-        m_expressionQueue.first()->interrupt();
-
-    m_expressionQueue.clear();
-    changeStatus(Cantor::Session::Done);
-}
-
-void MaximaSession::interrupt(MaximaExpression* expr)
-{
-    if(expr==m_expressionQueue.first())
+    if(!expressionQueue().isEmpty())
     {
-        qDebug()<<"interrupting " << expr->command();
-        disconnect(expr, nullptr, this, nullptr);
+        qDebug()<<"interrupting " << expressionQueue().first()->command();
+        if(m_process->state() != QProcess::NotRunning)
+        {
 #ifndef Q_OS_WIN
-        const int pid=m_process->pid();
-        kill(pid, SIGINT);
+            const int pid=m_process->pid();
+            kill(pid, SIGINT);
 #else
-      //TODO: interrupt the process on windows
+            ; //TODO: interrupt the process on windows
 #endif
+        }
+        expressionQueue().first()->interrupt();
+        expressionQueue().removeFirst();
+        foreach (Cantor::Expression* expression, expressionQueue())
+            expression->setStatus(Cantor::Expression::Done);
+        expressionQueue().clear();
+
         qDebug()<<"done interrupting";
-    }else
-    {
-        m_expressionQueue.removeAll(expr);
     }
+
+    changeStatus(Cantor::Session::Done);
+    m_cache.clear();
 }
 
 void MaximaSession::sendInputToProcess(const QString& input)
 {
-    qDebug()<<"WARNING: use this method only if you know what you're doing. Use evaluateExpression to run commands";
-    qDebug()<<"running "<<input;
     write(input);
 }
 
@@ -307,18 +284,18 @@ void MaximaSession::restartMaxima()
     {
         emit error(i18n("Maxima crashed. restarting..."));
         //remove the command that caused maxima to crash (to avoid infinite loops)
-        if(!m_expressionQueue.isEmpty())
-            m_expressionQueue.removeFirst();
+        if(!expressionQueue().isEmpty())
+            expressionQueue().removeFirst();
 
         m_justRestarted=true;
         QTimer::singleShot(1000, this, SLOT(restartsCooledDown()));
 
-        disconnect(m_process, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(restartMaxima()));
+        disconnect(m_process, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(restartMaxima()));
         login();
     }else
     {
-        if(!m_expressionQueue.isEmpty())
-            m_expressionQueue.removeFirst();
+        if(!expressionQueue().isEmpty())
+            expressionQueue().removeFirst();
         KMessageBox::error(nullptr, i18n("Maxima crashed twice within a short time. Stopping to try starting"), i18n("Error - Cantor"));
     }
 }
@@ -334,8 +311,7 @@ void MaximaSession::setTypesettingEnabled(bool enable)
     //we use the lisp command to set the variable, as those commands
     //don't mess with the labels and history
     const QString& val=QLatin1String((enable==true ? "t":"nil"));
-    Cantor::Expression* exp=evaluateExpression(QString::fromLatin1(":lisp(setf $display2d %1)").arg(val), Cantor::Expression::DeleteOnFinish);
-    exp->setInternal(true);
+    evaluateExpression(QString::fromLatin1(":lisp(setf $display2d %1)").arg(val), Cantor::Expression::DeleteOnFinish, true);
 
     Cantor::Session::setTypesettingEnabled(enable);
 }
@@ -361,6 +337,7 @@ QAbstractItemModel* MaximaSession::variableModel()
 }
 
 void MaximaSession::write(const QString& exp) {
-    qDebug()<<"sending expression to maxima process: " << exp << endl;
+    qDebug()<<"################################## EXPRESSION START ###############################################";
+    qDebug()<<"sending expression to maxima process: " << exp;
     m_process->write(exp.toUtf8());
 }
