@@ -26,7 +26,7 @@
 
 PythonHighlighter::PythonHighlighter(QObject* parent) : Cantor::DefaultHighlighter(parent)
 {
-    qDebug() << "PythonHighlighter construtor";
+    qDebug() << "PythonHighlighter constructor";
     addRule(QRegExp(QLatin1String("\\b\\w+(?=\\()")), functionFormat());
 
     //Code highlighting the different keywords
@@ -35,10 +35,6 @@ PythonHighlighter::PythonHighlighter(QObject* parent) : Cantor::DefaultHighlight
     addVariables(PythonKeywords::instance()->variables());
 }
 
-PythonHighlighter::~PythonHighlighter()
-{
-}
-
 void PythonHighlighter::highlightBlock(const QString &text)
 {
     if (skipHighlighting(text)) {
@@ -144,8 +140,12 @@ void PythonHighlighter::highlightBlock(const QString &text)
 void PythonHighlighter::updateHighlight()
 {
 
-    addVariables(m_variables);
-    rehighlight();
+    if (!m_variables.isEmpty())
+    {
+        addVariables(m_variables);
+        m_variables.clear();
+        rehighlight();
+    }
 }
 
 void PythonHighlighter::addVariable(const QString variable)
