@@ -28,15 +28,15 @@ class PythonHighlighter : public Cantor::DefaultHighlighter
     Q_OBJECT
 
   public:
-    PythonHighlighter(QObject* parent);
-    ~PythonHighlighter() override;
+    explicit PythonHighlighter(QObject* parent);
+    ~PythonHighlighter() override = default;
 
   public Q_SLOTS:
     void updateHighlight();
     void addVariable(const QString variable);
 
   protected:
-    void highlightBlock(const QString& text) Q_DECL_OVERRIDE;
+    void highlightBlock(const QString& text) override;
 
   private:
      QRegExp commentStartExpression;
