@@ -29,18 +29,29 @@
 #include <QDebug>
 #include <QDir>
 
+#include <QStandardPaths>
+#include <QDBusConnection>
+#include <QDBusInterface>
+#include <QDBusReply>
+
+#include <KProcess>
+
 #include <KDirWatch>
 
 #include <defaultvariablemodel.h>
 
 
-PythonSession::PythonSession(Cantor::Backend* backend) : Session(backend),
-m_variableModel(new Cantor::DefaultVariableModel(this))
-{
-}
-
-PythonSession::~PythonSession()
+PythonSession::PythonSession(Cantor::Backend* backend, int pythonVersion, const QString serverName, const QString DbusChannelName)
+    : Session(backend)
+    , m_variableModel(new Cantor::DefaultVariableModel(this))
+    , m_currentExpression(nullptr)
+    , m_pIface(nullptr)
+    , m_pProcess(nullptr)
+    , serverName(serverName)
+    , DbusChannelName(DbusChannelName)
 {
+    if (pythonVersion == 2)
+        PythonKeywords::instance()->python2Mode();
 }
 
 void PythonSession::login()
@@ -48,30 +59,49 @@ void PythonSession::login()
     qDebug()<<"login";
     emit loginStarted();
 
-    if(integratePlots())
+    // TODO: T6113, T6114
+    if (m_pProcess)
+        m_pProcess->deleteLater();
+
+    m_pProcess = new KProcess(this);
+    m_pProcess->setOutputChannelMode(KProcess::SeparateChannels);
+
+    (*m_pProcess) << QStandardPaths::findExecutable(serverName);
+
+    m_pProcess->start();
+
+    m_pProcess->waitForStarted();
+    m_pProcess->waitForReadyRead();
+    QTextStream stream(m_pProcess->readAllStandardOutput());
+
+    const QString& readyStatus = QString::fromLatin1("ready");
+    while (m_pProcess->state() == QProcess::Running)
     {
-        qDebug() << "integratePlots";
-
-        QString tempPath = QDir::tempPath();
-
-        QString pathOperations = tempPath;
-        pathOperations.prepend(QLatin1String("import os\nos.chdir('"));
-        pathOperations.append(QLatin1String("')\n"));
-
-        qDebug() << "Processing command to change chdir in Python. Command " << pathOperations.toLocal8Bit();
-
-        getPythonCommandOutput(pathOperations);
-
-        m_watch = new KDirWatch(this);
-        m_watch->setObjectName(QLatin1String("PythonDirWatch"));
-
-        m_watch->addDir(tempPath, KDirWatch::WatchFiles);
-
-        qDebug() << "addDir " <<  tempPath << "? " << m_watch->contains(QLatin1String(tempPath.toLocal8Bit()));
-
-        QObject::connect(m_watch, SIGNAL(created(QString)), SLOT(plotFileChanged(QString)));
+        const QString& rl = stream.readLine();
+        if (rl == readyStatus)
+            break;
     }
 
+    if (!QDBusConnection::sessionBus().isConnected())
+    {
+        qWarning() << "Can't connect to the D-Bus session bus.\n"
+                      "To start it, run: eval `dbus-launch --auto-syntax`";
+        return;
+    }
+
+    const QString& serviceName = DbusChannelName + QString::fromLatin1("-%1").arg(m_pProcess->pid());
+
+    m_pIface =  new QDBusInterface(serviceName,
+                                   QString::fromLatin1("/"), QString(), QDBusConnection::sessionBus());
+    if (!m_pIface->isValid())
+    {
+        qWarning() << QDBusConnection::sessionBus().lastError().message();
+        return;
+    }
+
+    m_pIface->call(QString::fromLatin1("login"));
+    m_pIface->call(QString::fromLatin1("setFilePath"), worksheetPath);
+
     const QStringList& scripts = autorunScripts();
     if(!scripts.isEmpty()){
         QString autorunScripts = scripts.join(QLatin1String("\n"));
@@ -80,27 +110,29 @@ void PythonSession::login()
 
     const QString& importerFile = QLatin1String(":py/import_default_modules.py");
 
-    evaluateExpression(fromSource(importerFile), Cantor::Expression::DeleteOnFinish);
+    evaluateExpression(fromSource(importerFile), Cantor::Expression::DeleteOnFinish, true);
 
     listVariables();
 
+    changeStatus(Session::Done);
     emit loginDone();
 }
 
 void PythonSession::logout()
 {
+    // TODO: T6113, T6114
+    m_pProcess->terminate();
+
     qDebug()<<"logout";
-
-    QDir removePlotFigures;
-    QListIterator<QString> i(m_listPlotName);
-
-    while(i.hasNext()){
-        removePlotFigures.remove(QLatin1String(i.next().toLocal8Bit().constData()));
-    }
+    changeStatus(Status::Disable);
 }
 
 void PythonSession::interrupt()
 {
+    // TODO: T6113, T6114
+    if (m_pProcess->pid())
+        m_pProcess->kill();
+
     qDebug()<<"interrupt";
 
     foreach(Cantor::Expression* e, m_runningExpressions)
@@ -110,10 +142,10 @@ void PythonSession::interrupt()
     changeStatus(Cantor::Session::Done);
 }
 
-Cantor::Expression* PythonSession::evaluateExpression(const QString& cmd, Cantor::Expression::FinishingBehavior behave)
+Cantor::Expression* PythonSession::evaluateExpression(const QString& cmd, Cantor::Expression::FinishingBehavior behave, bool internal)
 {
     qDebug() << "evaluating: " << cmd;
-    PythonExpression* expr = new PythonExpression(this);
+    PythonExpression* expr = new PythonExpression(this, internal);
 
     changeStatus(Cantor::Session::Running);
 
@@ -124,16 +156,6 @@ Cantor::Expression* PythonSession::evaluateExpression(const QString& cmd, Cantor
     return expr;
 }
 
-namespace
-{
-    int indentSize(const QString& command)
-    {
-        int contIdentationSpace = 0;
-        for(; command.at(contIdentationSpace).isSpace(); ++contIdentationSpace);
-        return contIdentationSpace;
-    }
-}
-
 void PythonSession::runExpression(PythonExpression* expr)
 {
     qDebug() << "run expression";
@@ -162,14 +184,6 @@ void PythonSession::runExpression(PythonExpression* expr)
             continue;
         }
 
-        if(!PythonKeywords::instance()->keywords().contains(firstLineWord) && !command.contains(QLatin1String("=")) &&
-           !command.endsWith(QLatin1String(":")) && !command.startsWith(QLatin1String(" "))){
-
-            commandProcessing += QLatin1String("print(") + command + QLatin1String(")\n");
-
-            continue;
-        }
-
         commandProcessing += command + QLatin1String("\n");
 
     }
@@ -182,16 +196,8 @@ void PythonSession::readExpressionOutput(const QString& commandProcessing)
     readOutput(commandProcessing);
 }
 
-void PythonSession::runClassOutputPython() const
-{
-    runPythonCommand(fromSource(QLatin1String(":py/init.py")));
-}
-
 void PythonSession::getPythonCommandOutput(const QString& commandProcessing)
 {
-    qDebug() << "Running python command" << commandProcessing;
-
-    runClassOutputPython();
     runPythonCommand(commandProcessing);
 
     m_output = getOutput();
@@ -322,23 +328,25 @@ void PythonSession::readOutput(const QString& commandProcessing)
     updateOutput();
 }
 
-void PythonSession::plotFileChanged(const QString& filename)
-{
-    qDebug() << "plotFileChanged filename:" << filename;
-
-    if ((m_currentExpression) && (filename.contains(QLatin1String("cantor-export-python-figure"))))
-    {
-         qDebug() << "Calling parsePlotFile";
-         m_currentExpression->parsePlotFile(filename);
-
-         m_listPlotName.append(filename);
-    }
-}
-
 void PythonSession::listVariables()
 {
-    QString listVariableCommand;
-    listVariableCommand += QLatin1String("print(globals())\n");
+    const QString& listVariableCommand = QLatin1String(
+        "try: \n"
+        "   import numpy \n"
+        "   __cantor_numpy_internal__ = numpy.get_printoptions()['threshold'] \n"
+        "   numpy.set_printoptions(threshold=100000000) \n"
+        "except ModuleNotFoundError: \n"
+        "   pass \n"
+
+        "print(globals()) \n"
+
+        "try: \n"
+        "   import numpy \n"
+        "   numpy.set_printoptions(threshold=__cantor_numpy_internal__) \n"
+        "   del __cantor_numpy_internal__ \n"
+        "except ModuleNotFoundError: \n"
+        "   pass \n"
+    );
 
     getPythonCommandOutput(listVariableCommand);
 
@@ -357,7 +365,7 @@ void PythonSession::listVariables()
         if(!first.startsWith(QLatin1String("'__")) && !first.startsWith(QLatin1String("__")) && !first.startsWith(QLatin1String("CatchOutPythonBackend'")) &&
            !first.startsWith(QLatin1String("errorPythonBackend'")) && !first.startsWith(QLatin1String("outputPythonBackend'")) &&
            !last.startsWith(QLatin1String(" class ")) && !last.startsWith(QLatin1String(" function ")) &&
-           !last.startsWith(QLatin1String(" module '") + first) /*skip imported modules*/ )
+           !last.startsWith(QLatin1String(" module '")) /*skip imported modules*/ )
         {
 
             m_variableModel->addVariable(parts.first().remove(QLatin1String("'")).simplified(), parts.last().simplified());
@@ -372,7 +380,7 @@ QSyntaxHighlighter* PythonSession::syntaxHighlighter(QObject* parent)
 {
     PythonHighlighter* highlighter = new PythonHighlighter(parent);
     QObject::connect(this, SIGNAL(updateHighlighter()), highlighter, SLOT(updateHighlight()));
-    QObject::connect(this, SIGNAL(newVariable(const QString)), highlighter, SLOT(addVariable(const QString)));
+    QObject::connect(this, SIGNAL(newVariable(QString)), highlighter, SLOT(addVariable(QString)));
 
     return highlighter;
 }
@@ -386,3 +394,34 @@ QAbstractItemModel* PythonSession::variableModel()
 {
     return m_variableModel;
 }
+
+void PythonSession::runPythonCommand(const QString& command) const
+{
+    // TODO: T6113, T6114
+    m_pIface->call(QString::fromLatin1("runPythonCommand"), command);
+}
+
+QString PythonSession::getOutput() const
+{
+    // TODO: T6113, T6114
+    const QDBusReply<QString>& reply = m_pIface->call(QString::fromLatin1("getOutput"));
+    if (reply.isValid())
+        return reply.value();
+
+    return reply.error().message();
+}
+
+QString PythonSession::getError() const
+{
+    // TODO: T6113, T6114
+    const QDBusReply<QString>& reply = m_pIface->call(QString::fromLatin1("getError"));
+    if (reply.isValid())
+        return reply.value();
+
+    return reply.error().message();
+}
+
+void PythonSession::setWorksheetPath(const QString& path)
+{
+    worksheetPath = path;
+}
