@@ -23,6 +23,7 @@
 #define _PYTHONSESSION_H
 
 #include "session.h"
+#include <cantor_pythonbackend_export.h>
 #include <QStringList>
 
 namespace Cantor {
@@ -31,36 +32,44 @@ class DefaultVariableModel;
 
 class PythonExpression;
 class KDirWatch;
+class QDBusInterface;
+class KProcess;
 
-class CANTOR_EXPORT PythonSession : public Cantor::Session
+class CANTOR_PYTHONBACKEND_EXPORT PythonSession : public Cantor::Session
 {
   Q_OBJECT
   public:
-    PythonSession(Cantor::Backend* backend);
-    ~PythonSession() override;
+    PythonSession(Cantor::Backend* backend, int pythonVersion, const QString serverName, const QString DbusChannelName);
+    ~PythonSession() override = default;
 
-    void login() Q_DECL_OVERRIDE;
-    void logout() Q_DECL_OVERRIDE;
+    void login() override;
+    void logout() override;
 
-    void interrupt() Q_DECL_OVERRIDE;
+    void interrupt() override;
     void runExpression(PythonExpression* expr);
 
-    Cantor::Expression* evaluateExpression(const QString& command, Cantor::Expression::FinishingBehavior behave) Q_DECL_OVERRIDE;
-    Cantor::CompletionObject* completionFor(const QString& command, int index=-1) Q_DECL_OVERRIDE;
-    QSyntaxHighlighter* syntaxHighlighter(QObject* parent) Q_DECL_OVERRIDE;
-    QAbstractItemModel* variableModel() Q_DECL_OVERRIDE;
+    Cantor::Expression* evaluateExpression(const QString& command, Cantor::Expression::FinishingBehavior behave = Cantor::Expression::FinishingBehavior::DoNotDelete, bool internal = false) override;
+    Cantor::CompletionObject* completionFor(const QString& command, int index=-1) override;
+    QSyntaxHighlighter* syntaxHighlighter(QObject* parent) override;
+    QAbstractItemModel* variableModel() override;
+    void setWorksheetPath(const QString& path) override;
 
     virtual bool integratePlots() const = 0;
     virtual QStringList autorunScripts() const = 0;
 
   private:
-    KDirWatch* m_watch;
-    QStringList m_listPlotName;
     Cantor::DefaultVariableModel* m_variableModel;
 
     QList<PythonExpression*> m_runningExpressions;
     PythonExpression* m_currentExpression;
 
+    QDBusInterface* m_pIface;
+    KProcess* m_pProcess;
+    QString serverName;
+    QString DbusChannelName;
+
+    QString worksheetPath;
+
   protected:
     QString m_output;
     QString m_error;
@@ -74,19 +83,17 @@ class CANTOR_EXPORT PythonSession : public Cantor::Session
     QString identifyVariableModule(const QString& command) const;
     bool identifyKeywords(const QString& command);
 
-    virtual void runPythonCommand(const QString& command) const = 0;
-    virtual QString getOutput() const = 0;
-    virtual QString getError() const = 0;
+    void runPythonCommand(const QString& command) const;
+    QString getOutput() const;
+    QString getError() const;
 
     virtual void readExpressionOutput(const QString& commandProcessing);
 
   protected:
-    void runClassOutputPython() const;
     void updateOutput();
 
   private Q_SLOTS:
     void readOutput(const QString& commandProcessing);
-    void plotFileChanged(const QString& filename);
     void expressionFinished();
 
   Q_SIGNALS:
