@@ -33,10 +33,6 @@ QalculateCompletionObject::QalculateCompletionObject(const QString& command, int
     setLine(command, index);
 }
 
-QalculateCompletionObject::~QalculateCompletionObject()
-{
-}
-
 void QalculateCompletionObject::fetchIdentifierType()
 {
     Variable* var = CALCULATOR->getVariable(identifier().toLatin1().data());
@@ -76,7 +72,7 @@ void QalculateCompletionObject::fetchCompletions()
     // does distinguish cases, and b) KCompletion::makeCompletion matches
     // case sensitive.
     foreach ( Unit* item, CALCULATOR->units ) {
-        //TODO: this is fugly...
+        //TODO: this is fugily...
         QString str(QLatin1String(item->name(true).c_str()));
         if ( str.startsWith(command(), Qt::CaseSensitive) ) {
             comp << str;
