@@ -31,21 +31,24 @@
 #include <QMimeDatabase>
 #include <QRegExp>
 
-SageExpression::SageExpression( Cantor::Session* session ) : Cantor::Expression(session),
+SageExpression::SageExpression( Cantor::Session* session, bool internal ) : Cantor::Expression(session, internal),
     m_isHelpRequest(false),
-    m_promptCount(0)
+    m_promptCount(0),
+    m_syntaxError(false)
 {
 }
 
 void SageExpression::evaluate()
 {
-    setStatus(Cantor::Expression::Computing);
     m_imagePath.clear();
 
     m_isHelpRequest=false;
 
-    //check if this is a ?command
-    if(command().startsWith(QLatin1Char('?'))||command().endsWith(QLatin1Char('?')))
+    //check if this is a ?command or help command
+    if( command().startsWith(QLatin1Char('?'))
+        || command().endsWith(QLatin1Char('?'))
+        || command().contains(QLatin1String("help("))
+    )
         m_isHelpRequest=true;
 
     //coun't how many newlines are in the command,
@@ -53,20 +56,25 @@ void SageExpression::evaluate()
     //each.
     m_promptCount=command().count(QLatin1Char('\n'))+2;
 
-    dynamic_cast<SageSession*>(session())->appendExpressionToQueue(this);
+    session()->enqueueExpression(this);
 }
 
 void SageExpression::interrupt()
 {
     qDebug()<<"interrupting";
-    dynamic_cast<SageSession*>(session())->sendSignalToProcess(2);
-    dynamic_cast<SageSession*>(session())->waitForNextPrompt();
 
     setStatus(Cantor::Expression::Interrupted);
 }
 
 void SageExpression::parseOutput(const QString& text)
 {
+    if (m_syntaxError)
+    {
+        setErrorMessage(i18n("Syntax Error"));
+        setStatus(Cantor::Expression::Error);
+        return;
+    }
+
     QString output=text;
     //remove carriage returns, we only use \n internally
     output.remove(QLatin1Char('\r'));
@@ -125,20 +133,21 @@ void SageExpression::parseOutput(const QString& text)
             qDebug()<<"got too many prompts";
 
         //if the output ends with an AlternativePrompt, this means that
-        //Sage is expecting additional input, allthough m_promptCount==0
+        //Sage is expecting additional input, although m_promptCount==0
         //indicates that all information has been passed to sage.
         //This means that the user has entered an invalid command.
         //interrupt it and show an error message
         if(endsWithAlternativePrompt)
         {
-            interrupt();
-            setErrorMessage(i18n("Syntax Error"));
-            setStatus(Cantor::Expression::Error);
-            return;
+            // Exit from sage additional input mode by closing the bracket
+            static_cast<SageSession*>(session())->sendInputToProcess(QLatin1String(")\n"));
+            m_syntaxError = true;
+        }
+        else
+        {
+            m_outputCache=m_outputCache.trimmed();
+            evalFinished();
         }
-
-        m_outputCache=m_outputCache.trimmed();
-        evalFinished();
     }
 
 }
@@ -146,7 +155,7 @@ void SageExpression::parseOutput(const QString& text)
 void SageExpression::parseError(const QString& text)
 {
     qDebug()<<"error";
-    setResult(new Cantor::TextResult(text));
+    setErrorMessage(text);
     setStatus(Cantor::Expression::Error);
 }
 
@@ -168,9 +177,9 @@ void SageExpression::evalFinished()
     qDebug()<<m_outputCache;
 
     //check if our image path contains a valid image that we can try to show
-    bool hasImage=!m_imagePath.isNull();
+    const bool hasImage=!m_imagePath.isNull();
 
-    if ( !hasImage ) //If this result contains a file, drop the text information
+    if (!m_outputCache.isEmpty())
     {
         Cantor::TextResult* result=nullptr;
 
@@ -188,12 +197,6 @@ void SageExpression::evalFinished()
         {
             stripped.remove( QRegExp( QLatin1String("<[a-zA-Z\\/][^>]*>") ) );
         }
-        else
-        {
-            //Replace < and > with their html code, so they won't be confused as html tags
-            stripped.replace( QLatin1Char('<') , QLatin1String("&lt;"));
-            stripped.replace( QLatin1Char('>') , QLatin1String("&gt;"));
-        }
         if (stripped.endsWith(QLatin1Char('\n')))
             stripped.chop(1);
 
@@ -215,16 +218,17 @@ void SageExpression::evalFinished()
         if(isLatex)
             result->setFormat(Cantor::TextResult::LatexFormat);
 
-        setResult(result);
+        addResult(result);
     }
-    else
+
+    if (hasImage)
     {
     QMimeDatabase db;
     QMimeType type = db.mimeTypeForUrl(QUrl::fromLocalFile(m_imagePath));
         if(type.inherits(QLatin1String("image/gif")))
-            setResult( new Cantor::AnimationResult(QUrl::fromLocalFile(m_imagePath ),i18n("Result of %1" , command() ) ) );
+            addResult( new Cantor::AnimationResult(QUrl::fromLocalFile(m_imagePath ),i18n("Result of %1" , command() ) ) );
         else
-            setResult( new Cantor::ImageResult( QUrl::fromLocalFile(m_imagePath ),i18n("Result of %1" , command() ) ) );
+            addResult( new Cantor::ImageResult( QUrl::fromLocalFile(m_imagePath ),i18n("Result of %1" , command() ) ) );
     }
     setStatus(Cantor::Expression::Done);
 }
