@@ -41,6 +41,7 @@
 #include <QProgressDialog>
 #include <KMessageBox>
 #include <KNS3/UploadDialog>
+#include <KXMLGUIFactory>
 
 #include <QElapsedTimer>
 #include <QFile>
@@ -69,41 +70,37 @@
 //A concrete implementation of the WorksheetAccesssInterface
 class WorksheetAccessInterfaceImpl : public Cantor::WorksheetAccessInterface
 {
-    //Q_OBJECT
+
   public:
     WorksheetAccessInterfaceImpl(QObject* parent, Worksheet* worksheet) :   WorksheetAccessInterface(parent),  m_worksheet(worksheet)
     {
         qDebug()<<"new worksheetaccess interface";
-        connect(worksheet, SIGNAL(sessionChanged()), this, SIGNAL(sessionChanged()));
-
+        connect(worksheet, SIGNAL(modified()), this, SIGNAL(modified()));
     }
 
-    ~WorksheetAccessInterfaceImpl() override
-    {
+    ~WorksheetAccessInterfaceImpl() override = default;
 
-    }
-
-    QByteArray saveWorksheetToByteArray() Q_DECL_OVERRIDE
+    QByteArray saveWorksheetToByteArray() override
     {
         return m_worksheet->saveToByteArray();
     }
 
-    void loadWorksheetFromByteArray(QByteArray* data) Q_DECL_OVERRIDE
+    void loadWorksheetFromByteArray(QByteArray* data) override
     {
         m_worksheet->load(data);
     }
 
-    Cantor::Session* session() Q_DECL_OVERRIDE
+    Cantor::Session* session() override
     {
         return m_worksheet->session();
     }
-//  public Q_SLOTS:
-  void evaluate() Q_DECL_OVERRIDE
+
+  void evaluate() override
     {
         m_worksheet->evaluate();
     }
 
-    void interrupt() Q_DECL_OVERRIDE
+    void interrupt() override
     {
         m_worksheet->interrupt();
     }
@@ -117,10 +114,9 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QVariantLi
     m_initProgressDlg(nullptr),
     m_showProgressDlg(true),
     m_showBackendHelp(nullptr),
-    m_statusBarBlocked(false)
+    m_statusBarBlocked(false),
+    m_sessionStatusCounter(0)
 {
-    qDebug()<<"Created a CantorPart";
-
     m_panelHandler=new Cantor::PanelPluginHandler(this);
     connect(m_panelHandler, SIGNAL(pluginsChanged()), this, SLOT(pluginsChanged()));
 
@@ -130,7 +126,7 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QVariantLi
     else
         backendName=args.first().toString();
 
-    foreach(const QVariant& arg, args)
+    for (const QVariant& arg : args)
     {
         if (arg.toString() == QLatin1String("--noprogress") )
         {
@@ -139,215 +135,224 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QVariantLi
         }
     }
 
-    Cantor::Backend* b=Cantor::Backend::createBackend(backendName);
+    Cantor::Backend* b=Cantor::Backend::getBackend(backendName);
     if(!b)
     {
         KMessageBox::error(parentWidget, i18n("Backend %1 is not installed", backendName), i18n("Error - Cantor"));
         setWidget(new QWidget(parentWidget));
-        //fake being modified so the shell won't try to reuse this part
-        ReadWritePart::setModified(true);
+        return;
+    }
+
+    if (b && !b->isEnabled() && backendName != QLatin1String("null"))
+    {
+        KMessageBox::information(parentWidget, i18n("There are some problems with the %1 backend,\n"\
+                                     "please check your configuration or install the needed packages.\n"
+                                     "You will only be able to view this worksheet.", backendName), i18n("Cantor"));
+        setWidget(new QWidget(parentWidget));
         return;
     }
 
     qDebug()<<"Backend "<<b->name()<<" offers extensions: "<<b->extensions();
 
+
+    auto* collection = actionCollection();
+
+    //central widget
     QWidget* widget = new QWidget(parentWidget);
     QVBoxLayout* layout = new QVBoxLayout(widget);
     m_worksheet=new Worksheet(b, widget);
     m_worksheetview=new WorksheetView(m_worksheet, widget);
     m_worksheetview->setEnabled(false); //disable input until the session has successfully logged in and emits the ready signal
     connect(m_worksheet, SIGNAL(modified()), this, SLOT(setModified()));
-    connect(m_worksheet, SIGNAL(showHelp(const QString&)), this, SIGNAL(showHelp(const QString&)));
-    connect(m_worksheet, SIGNAL(sessionChanged()), this, SLOT(worksheetSessionChanged()));
-    connect(actionCollection(), SIGNAL(inserted(QAction*)), m_worksheet,
-            SLOT(registerShortcut(QAction*)));
-
+    connect(m_worksheet, SIGNAL(showHelp(QString)), this, SIGNAL(showHelp(QString)));
+    connect(m_worksheet, SIGNAL(loaded()), this, SLOT(initialized()));
+    connect(collection, SIGNAL(inserted(QAction*)), m_worksheet, SLOT(registerShortcut(QAction*)));
     layout->addWidget(m_worksheetview);
-    // notify the part that this is our internal widget
     setWidget(widget);
 
-    Cantor::WorksheetAccessInterface* iface=new WorksheetAccessInterfaceImpl(this, m_worksheet);
+    //create WorksheetAccessInterface, used at the moment by LabPlot only to access Worksheet's API
+    Cantor::WorksheetAccessInterface* iface = new WorksheetAccessInterfaceImpl(this, m_worksheet);
     Q_UNUSED(iface);
 
-    // create our actions
-    m_worksheet->createActions(actionCollection());
+    //initialize actions
+    m_worksheet->createActions(collection);
 
-    KStandardAction::saveAs(this, SLOT(fileSaveAs()), actionCollection());
-    m_save = KStandardAction::save(this, SLOT(save()), actionCollection());
+    KStandardAction::saveAs(this, SLOT(fileSaveAs()), collection);
+    m_save = KStandardAction::save(this, SLOT(save()), collection);
     m_save->setPriority(QAction::LowPriority);
 
-    QAction * savePlain=new QAction(i18n("Save Plain Text"), actionCollection());
-    actionCollection()->addAction(QLatin1String("file_save_plain"), savePlain);
+    QAction* savePlain = new QAction(i18n("Save Plain Text"), collection);
+    collection->addAction(QLatin1String("file_save_plain"), savePlain);
     savePlain->setIcon(QIcon::fromTheme(QLatin1String("document-save")));
     connect(savePlain, SIGNAL(triggered()), this, SLOT(fileSavePlain()));
 
-    QAction * undo=KStandardAction::undo(m_worksheet, SIGNAL(undo()),
-                                        actionCollection());
+    QAction* undo = KStandardAction::undo(m_worksheet, SIGNAL(undo()), collection);
     undo->setPriority(QAction::LowPriority);
-    connect(m_worksheet, SIGNAL(undoAvailable(bool)),
-            undo, SLOT(setEnabled(bool)));
-    QAction * redo=KStandardAction::redo(m_worksheet, SIGNAL(redo()),
-                                        actionCollection());
+    connect(m_worksheet, SIGNAL(undoAvailable(bool)), undo, SLOT(setEnabled(bool)));
+    m_editActions.push_back(undo);
+
+    QAction* redo = KStandardAction::redo(m_worksheet, SIGNAL(redo()), collection);
     redo->setPriority(QAction::LowPriority);
-    connect(m_worksheet, SIGNAL(redoAvailable(bool)),
-            redo, SLOT(setEnabled(bool)));
+    connect(m_worksheet, SIGNAL(redoAvailable(bool)), redo, SLOT(setEnabled(bool)));
+    m_editActions.push_back(redo);
 
-    QAction * cut=KStandardAction::cut(m_worksheet, SIGNAL(cut()),
-                                      actionCollection());
+    QAction* cut = KStandardAction::cut(m_worksheet, SIGNAL(cut()), collection);
     cut->setPriority(QAction::LowPriority);
-    connect(m_worksheet, SIGNAL(cutAvailable(bool)),
-            cut, SLOT(setEnabled(bool)));
-    QAction * copy=KStandardAction::copy(m_worksheet, SIGNAL(copy()),
-                                        actionCollection());
-    copy->setPriority(QAction::LowPriority);
-    connect(m_worksheet, SIGNAL(copyAvailable(bool)),
-            copy, SLOT(setEnabled(bool)));
-    QAction * paste=KStandardAction::paste(m_worksheet, SIGNAL(paste()),
-                                          actionCollection());
-    paste->setPriority(QAction::LowPriority);
-    connect(m_worksheet, SIGNAL(pasteAvailable(bool)),
-            paste, SLOT(setEnabled(bool)));
+    connect(m_worksheet, SIGNAL(cutAvailable(bool)), cut, SLOT(setEnabled(bool)));
+    m_editActions.push_back(cut);
 
-    QAction * find=KStandardAction::find(this, SLOT(showSearchBar()),
-                                        actionCollection());
+    QAction* copy = KStandardAction::copy(m_worksheet, SIGNAL(copy()), collection);
+    copy->setPriority(QAction::LowPriority);
+    connect(m_worksheet, SIGNAL(copyAvailable(bool)), copy, SLOT(setEnabled(bool)));
+
+    QAction* paste = KStandardAction::paste(m_worksheet, SLOT(paste()), collection);
+    paste->setPriority(QAction::LowPriority);
+    connect(m_worksheet, SIGNAL(pasteAvailable(bool)), paste, SLOT(setEnabled(bool)));
+    m_editActions.push_back(paste);
+
+    QAction* find = KStandardAction::find(this, SLOT(showSearchBar()), collection);
     find->setPriority(QAction::LowPriority);
 
-    QAction * replace=KStandardAction::replace(this, SLOT(showExtendedSearchBar()),
-                                              actionCollection());
+    QAction* replace = KStandardAction::replace(this, SLOT(showExtendedSearchBar()), collection);
     replace->setPriority(QAction::LowPriority);
+    m_editActions.push_back(replace);
 
-    m_findNext = KStandardAction::findNext(this, SLOT(findNext()),
-                                           actionCollection());
+    m_findNext = KStandardAction::findNext(this, SLOT(findNext()), collection);
     m_findNext->setEnabled(false);
-    m_findPrev = KStandardAction::findPrev(this, SLOT(findPrev()),
-                                           actionCollection());
+
+    m_findPrev = KStandardAction::findPrev(this, SLOT(findPrev()), collection);
     m_findPrev->setEnabled(false);
 
-    QAction * latexExport=new QAction(i18n("Export to LaTeX"), actionCollection());
-    actionCollection()->addAction(QLatin1String("file_export_latex"), latexExport);
+    QAction* latexExport = new QAction(i18n("Export to LaTeX"), collection);
+    collection->addAction(QLatin1String("file_export_latex"), latexExport);
     latexExport->setIcon(QIcon::fromTheme(QLatin1String("document-export")));
     connect(latexExport, SIGNAL(triggered()), this, SLOT(exportToLatex()));
 
-    QAction * print = KStandardAction::print(this, SLOT(print()), actionCollection());
+    QAction* print = KStandardAction::print(this, SLOT(print()), collection);
     print->setPriority(QAction::LowPriority);
 
-    QAction * printPreview = KStandardAction::printPreview(this, SLOT(printPreview()), actionCollection());
+    QAction* printPreview = KStandardAction::printPreview(this, SLOT(printPreview()), collection);
     printPreview->setPriority(QAction::LowPriority);
 
-    KStandardAction::zoomIn(m_worksheetview, SLOT(zoomIn()), actionCollection());
-    KStandardAction::zoomOut(m_worksheetview, SLOT(zoomOut()), actionCollection());
+    KStandardAction::zoomIn(m_worksheetview, SLOT(zoomIn()), collection);
+    KStandardAction::zoomOut(m_worksheetview, SLOT(zoomOut()), collection);
+    KStandardAction::actualSize(m_worksheetview, SLOT(actualSize()), collection);
 
-    m_evaluate=new QAction(i18n("Evaluate Worksheet"), actionCollection());
-    actionCollection()->addAction(QLatin1String("evaluate_worksheet"), m_evaluate);
+    m_evaluate = new QAction(i18n("Evaluate Worksheet"), collection);
+    collection->addAction(QLatin1String("evaluate_worksheet"), m_evaluate);
     m_evaluate->setIcon(QIcon::fromTheme(QLatin1String("system-run")));
-    actionCollection()->setDefaultShortcut(m_evaluate, Qt::CTRL+Qt::Key_E);
+    collection->setDefaultShortcut(m_evaluate, Qt::CTRL+Qt::Key_E);
     connect(m_evaluate, SIGNAL(triggered()), this, SLOT(evaluateOrInterrupt()));
+    m_editActions.push_back(m_evaluate);
 
-    m_typeset=new KToggleAction(i18n("Typeset using LaTeX"), actionCollection());
+    m_typeset = new KToggleAction(i18n("Typeset using LaTeX"), collection);
     m_typeset->setChecked(Settings::self()->typesetDefault());
-    actionCollection()->addAction(QLatin1String("enable_typesetting"), m_typeset);
+    // Disable until login, because we use session command for this action
+    m_typeset->setEnabled(false);
+    collection->addAction(QLatin1String("enable_typesetting"), m_typeset);
     connect(m_typeset, SIGNAL(toggled(bool)), this, SLOT(enableTypesetting(bool)));
 
-    m_highlight=new KToggleAction(i18n("Syntax Highlighting"), actionCollection());
+    m_highlight = new KToggleAction(i18n("Syntax Highlighting"), collection);
     m_highlight->setChecked(Settings::self()->highlightDefault());
-    actionCollection()->addAction(QLatin1String("enable_highlighting"), m_highlight);
+    collection->addAction(QLatin1String("enable_highlighting"), m_highlight);
     connect(m_highlight, SIGNAL(toggled(bool)), m_worksheet, SLOT(enableHighlighting(bool)));
 
-    m_completion=new KToggleAction(i18n("Completion"), actionCollection());
+    m_completion = new KToggleAction(i18n("Completion"), collection);
     m_completion->setChecked(Settings::self()->completionDefault());
-    actionCollection()->addAction(QLatin1String("enable_completion"), m_completion);
+    collection->addAction(QLatin1String("enable_completion"), m_completion);
     connect(m_completion, SIGNAL(toggled(bool)), m_worksheet, SLOT(enableCompletion(bool)));
 
-    m_exprNumbering=new KToggleAction(i18n("Line Numbers"), actionCollection());
+    m_exprNumbering = new KToggleAction(i18n("Line Numbers"), collection);
     m_exprNumbering->setChecked(Settings::self()->expressionNumberingDefault());
-    actionCollection()->addAction(QLatin1String("enable_expression_numbers"), m_exprNumbering);
+    collection->addAction(QLatin1String("enable_expression_numbers"), m_exprNumbering);
     connect(m_exprNumbering, SIGNAL(toggled(bool)), m_worksheet, SLOT(enableExpressionNumbering(bool)));
 
-    m_animateWorksheet=new KToggleAction(i18n("Animate Worksheet"), actionCollection());
+    m_animateWorksheet = new KToggleAction(i18n("Animate Worksheet"), collection);
     m_animateWorksheet->setChecked(Settings::self()->animationDefault());
-    actionCollection()->addAction(QLatin1String("enable_animations"), m_animateWorksheet);
+    collection->addAction(QLatin1String("enable_animations"), m_animateWorksheet);
     connect(m_animateWorksheet, SIGNAL(toggled(bool)), m_worksheet, SLOT(enableAnimations(bool)));
 
-    QAction * restart=new QAction(i18n("Restart Backend"), actionCollection());
-    actionCollection()->addAction(QLatin1String("restart_backend"), restart);
+    QAction* restart = new QAction(i18n("Restart Backend"), collection);
+    collection->addAction(QLatin1String("restart_backend"), restart);
     restart->setIcon(QIcon::fromTheme(QLatin1String("system-reboot")));
     connect(restart, SIGNAL(triggered()), this, SLOT(restartBackend()));
+    m_editActions.push_back(restart);
 
-    QAction * evaluateCurrent=new QAction(i18n("Evaluate Entry"), actionCollection());
-    actionCollection()->addAction(QLatin1String("evaluate_current"),  evaluateCurrent);
-    actionCollection()->setDefaultShortcut(evaluateCurrent, Qt::SHIFT + Qt::Key_Return);
+    QAction* evaluateCurrent = new QAction(QIcon::fromTheme(QLatin1String("media-playback-start")), i18n("Evaluate Entry"), collection);
+    collection->addAction(QLatin1String("evaluate_current"),  evaluateCurrent);
+    collection->setDefaultShortcut(evaluateCurrent, Qt::SHIFT + Qt::Key_Return);
     connect(evaluateCurrent, SIGNAL(triggered()), m_worksheet, SLOT(evaluateCurrentEntry()));
+    m_editActions.push_back(evaluateCurrent);
 
-    QAction * insertCommandEntry=new QAction(i18n("Insert Command Entry"), actionCollection());
-    actionCollection()->addAction(QLatin1String("insert_command_entry"),  insertCommandEntry);
-    actionCollection()->setDefaultShortcut(insertCommandEntry, Qt::CTRL + Qt::Key_Return);
+    QAction* insertCommandEntry = new QAction(QIcon::fromTheme(QLatin1String("run-build")), i18n("Insert Command Entry"), collection);
+    collection->addAction(QLatin1String("insert_command_entry"),  insertCommandEntry);
+    collection->setDefaultShortcut(insertCommandEntry, Qt::CTRL + Qt::Key_Return);
     connect(insertCommandEntry, SIGNAL(triggered()), m_worksheet, SLOT(insertCommandEntry()));
+    m_editActions.push_back(insertCommandEntry);
 
-    QAction * insertTextEntry=new QAction(i18n("Insert Text Entry"), actionCollection());
-    actionCollection()->addAction(QLatin1String("insert_text_entry"),  insertTextEntry);
+    QAction* insertTextEntry = new QAction(QIcon::fromTheme(QLatin1String("draw-text")), i18n("Insert Text Entry"), collection);
+    collection->addAction(QLatin1String("insert_text_entry"),  insertTextEntry);
     connect(insertTextEntry, SIGNAL(triggered()), m_worksheet, SLOT(insertTextEntry()));
+    m_editActions.push_back(insertTextEntry);
 
-    QAction * insertLatexEntry=new QAction(i18n("Insert Latex Entry"), actionCollection());
-    actionCollection()->addAction(QLatin1String("insert_latex_entry"),  insertLatexEntry);
+#ifdef Discount_FOUND
+    QAction* insertMarkdownEntry = new QAction(QIcon::fromTheme(QLatin1String("text-x-markdown")), i18n("Insert Markdown Entry"), collection);
+    collection->addAction(QLatin1String("insert_markdown_entry"),  insertMarkdownEntry);
+    connect(insertMarkdownEntry, SIGNAL(triggered()), m_worksheet, SLOT(insertMarkdownEntry()));
+    m_editActions.push_back(insertMarkdownEntry);
+#endif
+
+#ifdef WITH_EPS
+    QAction* insertLatexEntry = new QAction(QIcon::fromTheme(QLatin1String("text-x-tex")), i18n("Insert Latex Entry"), collection);
+    collection->addAction(QLatin1String("insert_latex_entry"),  insertLatexEntry);
     connect(insertLatexEntry, SIGNAL(triggered()), m_worksheet, SLOT(insertLatexEntry()));
+    m_editActions.push_back(insertLatexEntry);
+#endif
 
-    QAction * insertPageBreakEntry=new QAction(i18n("Insert Page Break"), actionCollection());
-    actionCollection()->addAction(QLatin1String("insert_page_break_entry"), insertPageBreakEntry);
+    QAction* insertPageBreakEntry = new QAction(QIcon::fromTheme(QLatin1String("go-next-view-page")), i18n("Insert Page Break"), collection);
+    collection->addAction(QLatin1String("insert_page_break_entry"), insertPageBreakEntry);
     connect(insertPageBreakEntry, SIGNAL(triggered()), m_worksheet, SLOT(insertPageBreakEntry()));
+    m_editActions.push_back(insertPageBreakEntry);
 
-    QAction * insertImageEntry=new QAction(i18n("Insert Image"), actionCollection());
-    actionCollection()->addAction(QLatin1String("insert_image_entry"), insertImageEntry);
+    QAction* insertImageEntry = new QAction(QIcon::fromTheme(QLatin1String("image-x-generic")), i18n("Insert Image"), collection);
+    collection->addAction(QLatin1String("insert_image_entry"), insertImageEntry);
     connect(insertImageEntry, SIGNAL(triggered()), m_worksheet, SLOT(insertImageEntry()));
+    m_editActions.push_back(insertImageEntry);
 
-
-    /*
-    QAction * insertCommandEntryBefore=new QAction(i18n("Insert Command Entry Before"), actionCollection());
-    //insertCommandEntryBefore->setShortcut(Qt::CTRL + Qt::Key_Return);
-    actionCollection()->addAction("insert_command_entry_before",  insertCommandEntryBefore);
-    connect(insertCommandEntryBefore, SIGNAL(triggered()), m_worksheet, SLOT(insertCommandEntryBefore()));
-
-    QAction * insertTextEntryBefore=new QAction(i18n("Insert Text Entry Before"), actionCollection());
-    //insertTextEntryBefore->setShortcut(Qt::CTRL + Qt::Key_Return);
-    actionCollection()->addAction("insert_text_entry_before",  insertTextEntryBefore);
-    connect(insertTextEntryBefore, SIGNAL(triggered()), m_worksheet, SLOT(insertTextEntryBefore()));
-
-    QAction * insertPageBreakEntryBefore=new QAction(i18n("Insert Page Break Before"), actionCollection());
-    actionCollection()->addAction("insert_page_break_entry_before", insertPageBreakEntryBefore);
-    connect(insertPageBreakEntryBefore, SIGNAL(triggered()), m_worksheet, SLOT(insertPageBreakEntryBefore()));
-
-    QAction * insertImageEntryBefore=new QAction(i18n("Insert Image Entry Before"), actionCollection());
-    //insertTextEntryBefore->setShortcut(Qt::CTRL + Qt::Key_Return);
-    actionCollection()->addAction("insert_image_entry_before",  insertImageEntryBefore);
-    connect(insertImageEntryBefore, SIGNAL(triggered()), m_worksheet, SLOT(insertImageEntryBefore()));
-    */
-
-    QAction * removeCurrent=new QAction(i18n("Remove current Entry"), actionCollection());
-    actionCollection()->addAction(QLatin1String("remove_current"), removeCurrent);
-    actionCollection()->setDefaultShortcut(removeCurrent, Qt::ShiftModifier + Qt::Key_Delete);
+    QAction* removeCurrent = new QAction(QIcon::fromTheme(QLatin1String("edit-delete")), i18n("Remove current Entry"), collection);
+    collection->addAction(QLatin1String("remove_current"), removeCurrent);
+    collection->setDefaultShortcut(removeCurrent, Qt::ShiftModifier + Qt::Key_Delete);
     connect(removeCurrent, SIGNAL(triggered()), m_worksheet, SLOT(removeCurrentEntry()));
+    m_editActions.push_back(removeCurrent);
 
-    m_showBackendHelp=new QAction(i18n("Show %1 Help", b->name()) , actionCollection());
+    m_showBackendHelp = new QAction(i18n("Show %1 Help", b->name()) , collection);
     m_showBackendHelp->setIcon(QIcon::fromTheme(QLatin1String("help-contents")));
-    actionCollection()->addAction(QLatin1String("backend_help"), m_showBackendHelp);
+    collection->addAction(QLatin1String("backend_help"), m_showBackendHelp);
     connect(m_showBackendHelp, SIGNAL(triggered()), this, SLOT(showBackendHelp()));
 
-    QAction * publishWorksheet=new QAction(i18n("Publish Worksheet"), actionCollection());
+    // Disabled, because uploading to kde store from program don't work
+    // See https://phabricator.kde.org/T9980 for details
+    // If this situation will changed, then uncomment this action
+    /*
+    QAction* publishWorksheet = new QAction(i18n("Publish Worksheet"), collection);
     publishWorksheet->setIcon(QIcon::fromTheme(QLatin1String("get-hot-new-stuff")));
-    actionCollection()->addAction(QLatin1String("file_publish_worksheet"), publishWorksheet);
+    collection->addAction(QLatin1String("file_publish_worksheet"), publishWorksheet);
     connect(publishWorksheet, SIGNAL(triggered()), this, SLOT(publishWorksheet()));
+    */
 
-    KToggleAction* showEditor=new KToggleAction(i18n("Show Script Editor"), actionCollection());
+    KToggleAction* showEditor = new KToggleAction(i18n("Show Script Editor"), collection);
     showEditor->setChecked(false);
-    actionCollection()->addAction(QLatin1String("show_editor"), showEditor);
+    collection->addAction(QLatin1String("show_editor"), showEditor);
     connect(showEditor, SIGNAL(toggled(bool)), this, SLOT(showScriptEditor(bool)));
     showEditor->setEnabled(b->extensions().contains(QLatin1String("ScriptExtension")));
 
-    QAction * showCompletion=new QAction(i18n("Show Completion"), actionCollection());
-    actionCollection()->addAction(QLatin1String("show_completion"), showCompletion);
+    QAction* showCompletion = new QAction(i18n("Show Completion"), collection);
+    collection->addAction(QLatin1String("show_completion"), showCompletion);
     QList<QKeySequence> showCompletionShortcuts;
     showCompletionShortcuts << Qt::Key_Tab << Qt::CTRL + Qt::Key_Space;
-    actionCollection()->setDefaultShortcuts(showCompletion, showCompletionShortcuts);
+    collection->setDefaultShortcuts(showCompletion, showCompletionShortcuts);
     connect(showCompletion, SIGNAL(triggered()), m_worksheet, SLOT(showCompletion()));
+    m_editActions.push_back(showCompletion);
 
     // set our XML-UI resource file
     setXMLFile(QLatin1String("cantor_part.rc"));
@@ -358,7 +363,7 @@ CantorPart::CantorPart( QWidget *parentWidget, QObject *parent, const QVariantLi
     // we are not modified since we haven't done anything yet
     setModified(false);
 
-    worksheetSessionChanged();
+    initialized();
 }
 
 CantorPart::~CantorPart()
@@ -380,18 +385,26 @@ void CantorPart::setReadWrite(bool rw)
     ReadWritePart::setReadWrite(rw);
 }
 
+void CantorPart::setReadOnly()
+{
+    for (QAction* action : m_editActions)
+        action->setEnabled(false);
+
+    if (m_showBackendHelp)
+    {
+        m_showBackendHelp->setEnabled(false);
+        m_showBackendHelp->setVisible(false);
+    }
+}
+
 void CantorPart::setModified(bool modified)
 {
     // get a handle on our Save action and make sure it is valid
     if (!m_save)
         return;
 
-    // if so, we either enable or disable it based on the current
-    // state
-    if (modified)
-        m_save->setEnabled(true);
-    else
-        m_save->setEnabled(false);
+    // if so, we either enable or disable it based on the current state
+    m_save->setEnabled(modified);
 
     // in any event, we want our parent to do it's thing
     ReadWritePart::setModified(modified);
@@ -400,9 +413,7 @@ void CantorPart::setModified(bool modified)
 KAboutData& CantorPart::createAboutData()
 {
     // the non-i18n name here must be the same as the directory in
-    // which the part's rc file is installed ('partrcdir' in the
-    // Makefile)
-
+    // which the part's rc file is installed ('partrcdir' in the Makefile)
     static KAboutData about(QLatin1String("cantorpart"),
                      QLatin1String("Cantor"),
                      QLatin1String(CANTOR_VERSION),
@@ -434,6 +445,8 @@ bool CantorPart::openFile()
     if (rc) {
         qDebug()<< "Worksheet successfully loaded in " <<  (float)timer.elapsed()/1000 << " seconds).";
         updateCaption();
+        // We modified, but it we load file now, so no need in save option
+        setModified(false);
     }
 
     return rc;
@@ -461,12 +474,15 @@ void CantorPart::fileSaveAs()
     QString worksheetFilter = i18n("Cantor Worksheet (*.cws)");
     QString filter = worksheetFilter;
 
-    //if the backend supports scripts, also append their scriptFile endings to the filter
-    Cantor::Backend * const backend=m_worksheet->session()->backend();
-    if (backend->extensions().contains(QLatin1String("ScriptExtension")))
+    if (!m_worksheet->isReadOnly())
     {
-        Cantor::ScriptExtension* e=dynamic_cast<Cantor::ScriptExtension*>(backend->extension(QLatin1String("ScriptExtension")));
-        filter+=QLatin1String(";;")+e->scriptFileFilter();
+        //if the backend supports scripts, also append their scriptFile endings to the filter
+        Cantor::Backend * const backend=m_worksheet->session()->backend();
+        if (backend->extensions().contains(QLatin1String("ScriptExtension")))
+        {
+            Cantor::ScriptExtension* e=dynamic_cast<Cantor::ScriptExtension*>(backend->extension(QLatin1String("ScriptExtension")));
+            filter+=QLatin1String(";;")+e->scriptFileFilter();
+        }
     }
 
     QString selectedFilter;
@@ -530,21 +546,61 @@ void CantorPart::evaluateOrInterrupt()
 }
 void CantorPart::restartBackend()
 {
-    m_worksheet->session()->logout();
-    m_worksheet->session()->login();
+    bool restart = false;
+    if (Settings::self()->warnAboutSessionRestart())
+    {
+        KMessageBox::ButtonCode tmp;
+
+        // If we want the question box, but it is disable, then enable it
+        if (!KMessageBox::shouldBeShownYesNo(QLatin1String("WarnAboutSessionRestart"), tmp))
+            KMessageBox::enableMessage(QLatin1String("WarnAboutSessionRestart"));
+
+        const QString& name = m_worksheet->session()->backend()->name();
+        KMessageBox::ButtonCode rc = KMessageBox::questionYesNo(widget(),
+            i18n("All the available calculation results will be lost. Do you really want to restart %1?", name),
+            i18n("Restart %1?", name),
+            KStandardGuiItem::yes(),
+            KStandardGuiItem::no(),
+            QLatin1String("WarnAboutSessionRestart")
+        );
+
+        // Update setting's value
+        Settings::self()->setWarnAboutSessionRestart(
+            KMessageBox::shouldBeShownYesNo(QLatin1String("WarnAboutSessionRestart"), tmp));
+
+        restart = rc == KMessageBox::ButtonCode::Yes;
+    }
+    else
+    {
+        KMessageBox::ButtonCode rc;
+        KMessageBox::shouldBeShownYesNo(QLatin1String("WarnAboutSessionRestart"), rc);
+        restart = rc == KMessageBox::ButtonCode::Yes;
+    }
+
+    if (restart)
+    {
+        m_worksheet->session()->logout();
+        m_worksheet->session()->login();
+    }
 }
 
 void CantorPart::worksheetStatusChanged(Cantor::Session::Status status)
 {
     qDebug()<<"wsStatusChange"<<status;
+    unsigned int count = ++m_sessionStatusCounter;
     if(status==Cantor::Session::Running)
     {
-        m_evaluate->setText(i18n("Interrupt"));
-        m_evaluate->setShortcut(Qt::CTRL+Qt::Key_I);
-        m_evaluate->setIcon(QIcon::fromTheme(QLatin1String("dialog-close")));
-
-        setStatusMessage(i18n("Calculating..."));
-    }else
+        // Useless add a interrupt action without delay, because user physically can't interrupt fast commands
+        QTimer::singleShot(100, this, [this, count] () {
+            if(m_worksheet->session()->status() == Cantor::Session::Running && m_sessionStatusCounter == count)
+            {
+                m_evaluate->setText(i18n("Interrupt"));
+                m_evaluate->setShortcut(Qt::CTRL+Qt::Key_I);
+                m_evaluate->setIcon(QIcon::fromTheme(QLatin1String("dialog-close")));
+                setStatusMessage(i18n("Calculating..."));
+            }
+        });
+    }else if (status==Cantor::Session::Done)
     {
         m_evaluate->setText(i18n("Evaluate Worksheet"));
         m_evaluate->setShortcut(Qt::CTRL+Qt::Key_E);
@@ -561,26 +617,44 @@ void CantorPart::showSessionError(const QString& message)
     showImportantStatusMessage(i18n("Session Error: %1", message));
 }
 
-void CantorPart::worksheetSessionChanged()
-{
-    connect(m_worksheet->session(), SIGNAL(statusChanged(Cantor::Session::Status)), this, SLOT(worksheetStatusChanged(Cantor::Session::Status)));
-    connect(m_worksheet->session(), SIGNAL(loginStarted()),this, SLOT(worksheetSessionLoginStarted()));
-    connect(m_worksheet->session(), SIGNAL(loginDone()),this, SLOT(worksheetSessionLoginDone()));
-    connect(m_worksheet->session(), SIGNAL(error(const QString&)), this, SLOT(showSessionError(const QString&)));
-
-    loadAssistants();
-    m_panelHandler->setSession(m_worksheet->session());
-    adjustGuiToSession();
-    initialized();
-}
-
 void CantorPart::initialized()
 {
-    if (m_worksheet->isEmpty())
-        m_worksheet->appendCommandEntry();
+    if (!m_worksheet->isReadOnly())
+    {
+        connect(m_worksheet->session(), SIGNAL(statusChanged(Cantor::Session::Status)), this, SLOT(worksheetStatusChanged(Cantor::Session::Status)));
+        connect(m_worksheet->session(), SIGNAL(loginStarted()),this, SLOT(worksheetSessionLoginStarted()));
+        connect(m_worksheet->session(), SIGNAL(loginDone()),this, SLOT(worksheetSessionLoginDone()));
+        connect(m_worksheet->session(), SIGNAL(error(QString)), this, SLOT(showSessionError(QString)));
+
+        loadAssistants();
+        m_panelHandler->setSession(m_worksheet->session());
+        adjustGuiToSession();
+
+        // Don't set modification flag, if we add command entry in empty worksheet
+        const bool modified = this->isModified();
+        if (m_worksheet->isEmpty())
+            m_worksheet->appendCommandEntry();
+        setModified(modified);
+    }
+    else
+    {
+        setReadOnly();
+        // Clear assistants
+        for (KXMLGUIClient* client: childClients())
+        {
+            Cantor::Assistant* assistant = dynamic_cast<Cantor::Assistant*>(client);
+            if (assistant)
+            {
+                factory()->removeClient(client);
+                removeChildClient(client);
+                assistant->deleteLater();
+            }
+        }
+    }
 
     m_worksheetview->setEnabled(true);
     m_worksheetview->setFocus();
+
     setStatusMessage(i18n("Initialization complete"));
     updateCaption();
 }
@@ -592,6 +666,9 @@ void CantorPart::worksheetSessionLoginStarted() {
 
 void CantorPart::worksheetSessionLoginDone() {
     setStatusMessage(i18n("Ready"));
+
+    m_typeset->setEnabled(true);
+
     QApplication::restoreOverrideCursor();
 }
 
@@ -623,15 +700,16 @@ void CantorPart::updateCaption()
     if (filename.isEmpty())
         filename=i18n("Unnamed");
 
-    emit setCaption(filename, QIcon::fromTheme(m_worksheet->session()->backend()->icon()));
+    if (!m_worksheet->isReadOnly())
+        emit setCaption(filename, QIcon::fromTheme(m_worksheet->session()->backend()->icon()));
+    else
+        emit setCaption(filename+QLatin1Char(' ')+i18n("[read-only]"), QIcon());
 }
 
 void CantorPart::pluginsChanged()
 {
-    foreach(Cantor::PanelPlugin* plugin, m_panelHandler->plugins())
-    {
+    for (auto* plugin : m_panelHandler->plugins())
         connect(plugin, SIGNAL(requestRunCommand(QString)), this, SLOT(runCommand(QString)));
-    }
 }
 
 void CantorPart::loadAssistants()
@@ -639,12 +717,11 @@ void CantorPart::loadAssistants()
     qDebug()<<"loading assistants...";
 
     QStringList assistantDirs;
-    foreach(const QString &dir, QCoreApplication::libraryPaths()){
+    for (const QString& dir : QCoreApplication::libraryPaths())
         assistantDirs << dir + QDir::separator() + QLatin1String("cantor/assistants");
-    }
 
     QPluginLoader loader;
-    foreach(const QString &dir, assistantDirs){
+    for (const QString& dir : assistantDirs) {
 
         qDebug() << "dir: " << dir;
         QStringList assistants;
@@ -652,7 +729,7 @@ void CantorPart::loadAssistants()
 
         assistants = assistantDir.entryList();
 
-        foreach (const QString &assistant, assistants){
+        for (const QString& assistant : assistants) {
             if (assistant==QLatin1String(".") || assistant==QLatin1String(".."))
                 continue;
 
@@ -672,19 +749,18 @@ void CantorPart::loadAssistants()
             plugin->setPluginInfo(info);
             plugin->setBackend(backend);
 
-            qDebug()<<"plugin "<<info.name()<<" requires "<<plugin->requiredExtensions();
             bool supported=true;
-            foreach(const QString& req, plugin->requiredExtensions())
+            for (const QString& req : plugin->requiredExtensions())
                 supported=supported && backend->extensions().contains(req);
 
-            qDebug()<<"plugin "<<info.name()<<" is "<<(supported ? "":" not ")<<" supported by "<<backend->name();
-
             if(supported)
             {
+                qDebug() << "plugin " << info.name() << " is supported by " << backend->name() << ", requires extensions " << plugin->requiredExtensions();
                 plugin->initActions();
                 connect(plugin, SIGNAL(requested()), this, SLOT(runAssistant()));
             }else
             {
+                qDebug() << "plugin " << info.name() << " is not supported by "<<backend->name();
                 removeChildClient(plugin);
                 plugin->deleteLater();
             }
@@ -759,12 +835,13 @@ void CantorPart::searchBarDeleted()
 
 void CantorPart::adjustGuiToSession()
 {
+    Cantor::Backend::Capabilities capabilities = m_worksheet->session()->backend()->capabilities();
 #ifdef WITH_EPS
-    m_typeset->setVisible(m_worksheet->session()->backend()->capabilities().testFlag(Cantor::Backend::LaTexOutput));
+    m_typeset->setVisible(capabilities.testFlag(Cantor::Backend::LaTexOutput));
 #else
     m_typeset->setVisible(false);
 #endif
-    m_completion->setVisible(m_worksheet->session()->backend()->capabilities().testFlag(Cantor::Backend::Completion));
+    m_completion->setVisible(capabilities.testFlag(Cantor::Backend::Completion));
 
     //this is 0 on the first call
     if(m_showBackendHelp)
@@ -833,7 +910,7 @@ void CantorPart::showScriptEditor(bool show)
             return;
         }
         m_scriptEditor=new ScriptEditorWidget(scriptE->scriptFileFilter(), scriptE->highlightingMode(), widget()->window());
-        connect(m_scriptEditor, SIGNAL(runScript(const QString&)), this, SLOT(runScript(const QString&)));
+        connect(m_scriptEditor, SIGNAL(runScript(QString)), this, SLOT(runScript(QString)));
         connect(m_scriptEditor, SIGNAL(destroyed()), this, SLOT(scriptEditorClosed()));
         m_scriptEditor->show();
     }else
