@@ -22,6 +22,7 @@
 #define CANTORPART_H
 
 #include <QPointer>
+#include <QVector>
 
 #include <KParts/ReadWritePart>
 #include <lib/session.h>
@@ -67,12 +68,12 @@ public:
      * A shell will use this to inform this Part if it should act
      * read-only
      */
-    void setReadWrite(bool rw) Q_DECL_OVERRIDE;
+    void setReadWrite(bool rw) override;
 
     /**
      * Reimplemented to disable and enable Save action
      */
-    void setModified(bool modified) Q_DECL_OVERRIDE;
+    void setModified(bool modified) override;
 
     KAboutData& createAboutData();
 
@@ -86,23 +87,25 @@ protected:
     /**
      * This must be implemented by each part
      */
-    bool openFile() Q_DECL_OVERRIDE;
+    bool openFile() override;
 
     /**
      * This must be implemented by each read-write part
      */
-    bool saveFile() Q_DECL_OVERRIDE;
+    bool saveFile() override;
 
     /**
      * Called when this part becomes the active one,
      * or if it looses activity
      **/
-    void guiActivateEvent( KParts::GUIActivateEvent * event ) Q_DECL_OVERRIDE;
+    void guiActivateEvent( KParts::GUIActivateEvent * event ) override;
 
 
     void loadAssistants();
     void adjustGuiToSession();
 
+    void setReadOnly();
+
 protected Q_SLOTS:
     void fileSaveAs();
     void fileSavePlain();
@@ -116,7 +119,6 @@ protected Q_SLOTS:
 
     void worksheetStatusChanged(Cantor::Session::Status stauts);
     void showSessionError(const QString& error);
-    void worksheetSessionChanged();
     void worksheetSessionLoginStarted();
     void worksheetSessionLoginDone();
     void initialized();
@@ -172,9 +174,11 @@ private:
     KToggleAction* m_exprNumbering;
     KToggleAction* m_animateWorksheet;
     QAction * m_showBackendHelp;
+    QVector<QAction*> m_editActions;
 
     QString m_cachedStatusMessage;
     bool m_statusBarBlocked;
+    unsigned int m_sessionStatusCounter;
 };
 
 #endif // CANTORPART_H
