@@ -17,6 +17,7 @@
     ---
     Copyright (C) 2009 Alexander Rieder <alexanderrieder@gmail.com>
     Copyright (C) 2012 Martin Kuettler <martin.kuettler@gmail.com>
+    Copyright (C) 2018 Alexander Semke <alexander.semke@web.de>
  */
 
 #include "commandentry.h"
@@ -28,11 +29,14 @@
 #include "lib/syntaxhelpobject.h"
 #include "lib/session.h"
 
-#include <QToolTip>
 #include <QApplication>
-#include <QDesktopWidget>
-
 #include <QDebug>
+#include <QDesktopWidget>
+#include <QFontDialog>
+#include <QTimer>
+#include <QToolTip>
+#include <QPropertyAnimation>
+
 #include <KLocalizedString>
 #include <KColorScheme>
 
@@ -40,20 +44,49 @@ const QString CommandEntry::Prompt=QLatin1String(">>> ");
 const double CommandEntry::HorizontalSpacing = 4;
 const double CommandEntry::VerticalSpacing = 4;
 
+static const int colorsCount = 26;
+static QColor colors[colorsCount] = {QColor(255,255,255), QColor(0,0,0),
+							QColor(192,0,0), QColor(255,0,0), QColor(255,192,192), //red
+							QColor(0,192,0), QColor(0,255,0), QColor(192,255,192), //green
+							QColor(0,0,192), QColor(0,0,255), QColor(192,192,255), //blue
+							QColor(192,192,0), QColor(255,255,0), QColor(255,255,192), //yellow
+							QColor(0,192,192), QColor(0,255,255), QColor(192,255,255), //cyan
+							QColor(192,0,192), QColor(255,0,255), QColor(255,192,255), //magenta
+							QColor(192,88,0), QColor(255,128,0), QColor(255,168,88), //orange
+							QColor(128,128,128), QColor(160,160,160), QColor(195,195,195) //grey
+							};
+
+
 CommandEntry::CommandEntry(Worksheet* worksheet) : WorksheetEntry(worksheet),
     m_promptItem(new WorksheetTextItem(this, Qt::NoTextInteraction)),
     m_commandItem(new WorksheetTextItem(this, Qt::TextEditorInteraction)),
-    m_resultItem(nullptr),
     m_errorItem(nullptr),
     m_expression(nullptr),
     m_completionObject(nullptr),
-    m_syntaxHelpObject(nullptr)
+    m_syntaxHelpObject(nullptr),
+    m_evaluationOption(DoNothing),
+    m_menusInitialized(false),
+    m_backgroundColorActionGroup(nullptr),
+    m_backgroundColorMenu(nullptr),
+    m_textColorActionGroup(nullptr),
+    m_textColorMenu(nullptr),
+    m_fontMenu(nullptr)
 {
 
     m_promptItem->setPlainText(Prompt);
     m_promptItem->setItemDragable(true);
     m_commandItem->enableCompletion(true);
 
+    KColorScheme scheme = KColorScheme(QPalette::Normal, KColorScheme::View);
+    m_commandItem->setBackgroundColor(scheme.background(KColorScheme::AlternateBackground).color());
+
+    m_promptItemAnimation = new QPropertyAnimation(m_promptItem, "opacity");
+    m_promptItemAnimation->setDuration(600);
+    m_promptItemAnimation->setStartValue(1);
+    m_promptItemAnimation->setKeyValueAt(0.5, 0);
+    m_promptItemAnimation->setEndValue(1);
+    connect(m_promptItemAnimation, &QPropertyAnimation::finished, this, &CommandEntry::animatePromptItem);
+
     connect(m_commandItem, &WorksheetTextItem::tabPressed, this, &CommandEntry::showCompletion);
     connect(m_commandItem, &WorksheetTextItem::backtabPressed, this, &CommandEntry::selectPreviousCompletion);
     connect(m_commandItem, &WorksheetTextItem::applyCompletion, this, &CommandEntry::applySelectedCompletion);
@@ -76,8 +109,178 @@ int CommandEntry::type() const
     return Type;
 }
 
+void CommandEntry::initMenus() {
+    //background color
+	const QString colorNames[colorsCount] = {i18n("White"), i18n("Black"),
+							i18n("Dark Red"), i18n("Red"), i18n("Light Red"),
+							i18n("Dark Green"), i18n("Green"), i18n("Light Green"),
+							i18n("Dark Blue"), i18n("Blue"), i18n("Light Blue"),
+							i18n("Dark Yellow"), i18n("Yellow"), i18n("Light Yellow"),
+							i18n("Dark Cyan"), i18n("Cyan"), i18n("Light Cyan"),
+							i18n("Dark Magenta"), i18n("Magenta"), i18n("Light Magenta"),
+							i18n("Dark Orange"), i18n("Orange"), i18n("Light Orange"),
+							i18n("Dark Grey"), i18n("Grey"), i18n("Light Grey")
+							};
+
+    //background color
+    m_backgroundColorActionGroup = new QActionGroup(this);
+	m_backgroundColorActionGroup->setExclusive(true);
+	connect(m_backgroundColorActionGroup, &QActionGroup::triggered, this, &CommandEntry::backgroundColorChanged);
+
+    m_backgroundColorMenu = new QMenu(i18n("Background Color"));
+    m_backgroundColorMenu->setIcon(QIcon::fromTheme(QLatin1String("format-fill-color")));
+
+	QPixmap pix(16,16);
+	QPainter p(&pix);
+	for (int i=0; i<colorsCount; ++i) {
+		p.fillRect(pix.rect(), colors[i]);
+		QAction* action = new QAction(QIcon(pix), colorNames[i], m_backgroundColorActionGroup);
+		action->setCheckable(true);
+		m_backgroundColorMenu->addAction(action);
+	}
+
+	//text color
+    m_textColorActionGroup = new QActionGroup(this);
+	m_textColorActionGroup->setExclusive(true);
+	connect(m_textColorActionGroup, &QActionGroup::triggered, this, &CommandEntry::textColorChanged);
+
+    m_textColorMenu = new QMenu(i18n("Text Color"));
+    m_textColorMenu->setIcon(QIcon::fromTheme(QLatin1String("format-text-color")));
+
+	for (int i=0; i<colorsCount; ++i) {
+		p.fillRect(pix.rect(), colors[i]);
+		QAction* action = new QAction(QIcon(pix), colorNames[i], m_textColorActionGroup);
+		action->setCheckable(true);
+		m_textColorMenu->addAction(action);
+	}
+
+	//font
+	m_fontMenu = new QMenu(i18n("Font"));
+    m_fontMenu->setIcon(QIcon::fromTheme(QLatin1String("preferences-desktop-font")));
+
+    QAction* action = new QAction(QIcon::fromTheme(QLatin1String("format-text-bold")), i18n("Bold"));
+    action->setCheckable(true);
+    connect(action, &QAction::triggered, this, &CommandEntry::fontBoldTriggered);
+    m_fontMenu->addAction(action);
+
+    action = new QAction(QIcon::fromTheme(QLatin1String("format-text-italic")), i18n("Italic"));
+    action->setCheckable(true);
+    connect(action, &QAction::triggered, this, &CommandEntry::fontItalicTriggered);
+    m_fontMenu->addAction(action);
+    m_fontMenu->addSeparator();
+
+    action = new QAction(QIcon::fromTheme(QLatin1String("format-font-size-less")), i18n("Increase Size"));
+    connect(action, &QAction::triggered, this, &CommandEntry::fontIncreaseTriggered);
+    m_fontMenu->addAction(action);
+
+    action = new QAction(QIcon::fromTheme(QLatin1String("format-font-size-more")), i18n("Decrease Size"));
+    connect(action, &QAction::triggered, this, &CommandEntry::fontDecreaseTriggered);
+    m_fontMenu->addAction(action);
+    m_fontMenu->addSeparator();
+
+    action = new QAction(QIcon::fromTheme(QLatin1String("preferences-desktop-font")), i18n("Select"));
+    connect(action, &QAction::triggered, this, &CommandEntry::fontSelectTriggered);
+    m_fontMenu->addAction(action);
+
+    m_menusInitialized = true;
+}
+
+void CommandEntry::backgroundColorChanged(QAction* action) {
+	int index = m_backgroundColorActionGroup->actions().indexOf(action);
+	if (index == -1 || index>=colorsCount)
+		index = 0;
+
+    m_commandItem->setBackgroundColor(colors[index]);
+}
+
+void CommandEntry::textColorChanged(QAction* action) {
+	int index = m_textColorActionGroup->actions().indexOf(action);
+	if (index == -1 || index>=colorsCount)
+		index = 0;
+
+    m_commandItem->setDefaultTextColor(colors[index]);
+}
+
+void CommandEntry::fontBoldTriggered()
+{
+    QAction* action = static_cast<QAction*>(QObject::sender());
+    QFont font = m_commandItem->font();
+    font.setBold(action->isChecked());
+    m_commandItem->setFont(font);
+}
+
+void CommandEntry::fontItalicTriggered()
+{
+    QAction* action = static_cast<QAction*>(QObject::sender());
+    QFont font = m_commandItem->font();
+    font.setItalic(action->isChecked());
+    m_commandItem->setFont(font);
+}
+
+void CommandEntry::fontIncreaseTriggered()
+{
+    QFont font = m_commandItem->font();
+    const int currentSize = font.pointSize();
+    QFontDatabase fdb;
+    QList<int> sizes = fdb.pointSizes(font.family(), font.styleName());
+
+    for (int i = 0; i < sizes.size(); ++i)
+    {
+        const int size = sizes.at(i);
+        if (currentSize == size)
+        {
+            if (i + 1 < sizes.size())
+            {
+                font.setPointSize(sizes.at(i+1));
+                m_commandItem->setFont(font);
+            }
+
+            break;
+        }
+    }
+}
+
+void CommandEntry::fontDecreaseTriggered()
+{
+    QFont font = m_commandItem->font();
+    const int currentSize = font.pointSize();
+    QFontDatabase fdb;
+    QList<int> sizes = fdb.pointSizes(font.family(), font.styleName());
+
+    for (int i = 0; i < sizes.size(); ++i)
+    {
+        const int size = sizes.at(i);
+        if (currentSize == size)
+        {
+            if (i - 1 >= 0)
+            {
+                font.setPointSize(sizes.at(i-1));
+                m_commandItem->setFont(font);
+            }
+
+            break;
+        }
+    }
+}
+
+void CommandEntry::fontSelectTriggered()
+{
+    bool ok;
+    QFont font = QFontDialog::getFont(&ok, m_commandItem->font(), nullptr);
+
+    if (ok)
+        m_commandItem->setFont(font);
+}
+
 void CommandEntry::populateMenu(QMenu* menu, QPointF pos)
 {
+    if (!m_menusInitialized)
+        initMenus();
+
+    menu->addMenu(m_backgroundColorMenu);
+    menu->addMenu(m_textColorMenu);
+    menu->addMenu(m_fontMenu);
+    menu->addSeparator();
     WorksheetEntry::populateMenu(menu, pos);
 }
 
@@ -132,7 +335,7 @@ void CommandEntry::setExpression(Cantor::Expression* expr)
         m_expression->deleteLater();
         }*/
 
-    // Delete any previus error
+    // Delete any previous error
     if(m_errorItem)
     {
         m_errorItem->deleteLater();
@@ -145,16 +348,18 @@ void CommandEntry::setExpression(Cantor::Expression* expr)
     }
     m_informationItems.clear();
 
-    m_expression = nullptr;
     // Delete any previous result
-    removeResult();
+    clearResultItems();
 
-    m_expression=expr;
+    m_expression = expr;
 
     connect(expr, SIGNAL(gotResult()), this, SLOT(updateEntry()));
+    connect(expr, SIGNAL(resultsCleared()), this, SLOT(clearResultItems()));
+    connect(expr, SIGNAL(resultRemoved(int)), this, SLOT(removeResultItem(int)));
+    connect(expr, SIGNAL(resultReplaced(int)), this, SLOT(replaceResultItem(int)));
     connect(expr, SIGNAL(idChanged()), this, SLOT(updatePrompt()));
     connect(expr, SIGNAL(statusChanged(Cantor::Expression::Status)), this, SLOT(expressionChangedStatus(Cantor::Expression::Status)));
-    connect(expr, SIGNAL(needsAdditionalInformation(const QString&)), this, SLOT(showAdditionalInformationPrompt(const QString&)));
+    connect(expr, SIGNAL(needsAdditionalInformation(QString)), this, SLOT(showAdditionalInformationPrompt(QString)));
     connect(expr, SIGNAL(statusChanged(Cantor::Expression::Status)), this, SLOT(updatePrompt()));
 
     updatePrompt();
@@ -164,10 +369,8 @@ void CommandEntry::setExpression(Cantor::Expression* expr)
         worksheet()->gotResult(expr);
         updateEntry();
     }
-    if(expr->status()!=Cantor::Expression::Computing)
-    {
-        expressionChangedStatus(expr->status());
-    }
+
+    expressionChangedStatus(expr->status());
 }
 
 Cantor::Expression* CommandEntry::expression()
@@ -192,6 +395,39 @@ void CommandEntry::setContent(const QDomElement& content, const KZip& file)
     LoadedExpression* expr=new LoadedExpression( worksheet()->session() );
     expr->loadFromXml(content, file);
 
+    //background color
+    QDomElement backgroundElem = content.firstChildElement(QLatin1String("Background"));
+    if (!backgroundElem.isNull())
+    {
+        QColor color;
+        color.setRed(backgroundElem.attribute(QLatin1String("red")).toInt());
+        color.setGreen(backgroundElem.attribute(QLatin1String("green")).toInt());
+        color.setBlue(backgroundElem.attribute(QLatin1String("blue")).toInt());
+        m_commandItem->setBackgroundColor(color);
+    }
+
+    //text properties
+    QDomElement textElem = content.firstChildElement(QLatin1String("Text"));
+    if (!textElem.isNull())
+    {
+        //text color
+        QDomElement colorElem = textElem.firstChildElement(QLatin1String("Color"));
+        QColor color;
+        color.setRed(colorElem.attribute(QLatin1String("red")).toInt());
+        color.setGreen(colorElem.attribute(QLatin1String("green")).toInt());
+        color.setBlue(colorElem.attribute(QLatin1String("blue")).toInt());
+        m_commandItem->setDefaultTextColor(color);
+
+        //font properties
+        QDomElement fontElem = textElem.firstChildElement(QLatin1String("Font"));
+        QFont font;
+        font.setFamily(fontElem.attribute(QLatin1String("family")));
+        font.setPointSize(fontElem.attribute(QLatin1String("pointSize")).toInt());
+        font.setWeight(fontElem.attribute(QLatin1String("weight")).toInt());
+        font.setItalic(fontElem.attribute(QLatin1String("italic")).toInt());
+        m_commandItem->setFont(font);
+    }
+
     setExpression(expr);
 }
 
@@ -245,18 +481,60 @@ QString CommandEntry::toPlain(const QString& commandSep, const QString& commentS
 
 QDomElement CommandEntry::toXml(QDomDocument& doc, KZip* archive)
 {
+    QDomElement exprElem = doc.createElement( QLatin1String("Expression") );
+    QDomElement cmdElem = doc.createElement( QLatin1String("Command") );
+    cmdElem.appendChild(doc.createTextNode( command() ));
+    exprElem.appendChild(cmdElem);
+
+    // save results of the expression if they exist
     if (expression())
+        for (Cantor::Result * const result: expression()->results())
+        {
+            const QDomElement& resultElem = result->toXml(doc);
+            exprElem.appendChild(resultElem);
+
+            if (archive)
+                result->saveAdditionalData(archive);
+        }
+
+    //save the background color if it differs from the default one
+    const QColor& backgroundColor = m_commandItem->backgroundColor();
+    KColorScheme scheme = KColorScheme(QPalette::Normal, KColorScheme::View);
+    if (backgroundColor != scheme.background(KColorScheme::AlternateBackground).color())
     {
-        if ( archive )
-            expression()->saveAdditionalData( archive );
-        return expression()->toXml(doc);
+        QDomElement colorElem = doc.createElement( QLatin1String("Background") );
+        colorElem.setAttribute(QLatin1String("red"), QString::number(backgroundColor.red()));
+        colorElem.setAttribute(QLatin1String("green"), QString::number(backgroundColor.green()));
+        colorElem.setAttribute(QLatin1String("blue"), QString::number(backgroundColor.blue()));
+        exprElem.appendChild(colorElem);
     }
-    QDomElement expr=doc.createElement( QLatin1String("Expression") );
-    QDomElement cmd=doc.createElement( QLatin1String("Command") );
-    QDomText cmdText=doc.createTextNode( command() );
-    cmd.appendChild( cmdText );
-    expr.appendChild( cmd );
-    return expr;
+
+    //save the text properties if they differ from default values
+    const QFont& font = m_commandItem->font();
+    if (font != QFontDatabase::systemFont(QFontDatabase::FixedFont))
+    {
+        QDomElement textElem = doc.createElement(QLatin1String("Text"));
+
+        //font properties
+        QDomElement fontElem = doc.createElement(QLatin1String("Font"));
+        fontElem.setAttribute(QLatin1String("family"), font.family());
+        fontElem.setAttribute(QLatin1String("pointSize"), QString::number(font.pointSize()));
+        fontElem.setAttribute(QLatin1String("weight"), QString::number(font.weight()));
+        fontElem.setAttribute(QLatin1String("italic"), QString::number(font.italic()));
+        textElem.appendChild(fontElem);
+
+        //text color
+        const QColor& textColor = m_commandItem->defaultTextColor();
+        QDomElement colorElem = doc.createElement( QLatin1String("Color") );
+        colorElem.setAttribute(QLatin1String("red"), QString::number(textColor.red()));
+        colorElem.setAttribute(QLatin1String("green"), QString::number(textColor.green()));
+        colorElem.setAttribute(QLatin1String("blue"), QString::number(textColor.blue()));
+        textElem.appendChild(colorElem);
+
+        exprElem.appendChild(textElem);
+    }
+
+    return exprElem;
 }
 
 QString CommandEntry::currentLine()
@@ -292,7 +570,7 @@ bool CommandEntry::evaluate(EvaluationOption evalOp)
     m_evaluationOption = evalOp;
 
     if(cmd.isEmpty()) {
-        removeResult();
+        removeResults();
         foreach(WorksheetTextItem* item, m_informationItems) {
             item->deleteLater();
         }
@@ -322,61 +600,83 @@ void CommandEntry::interruptEvaluation()
 void CommandEntry::updateEntry()
 {
     qDebug() << "update Entry";
-    Cantor::Expression *expr = expression();
-    if (expr == nullptr || expr->result() == nullptr)
+    Cantor::Expression* expr = expression();
+    if (expr == nullptr || expr->results().isEmpty())
         return;
 
-    if (expr->result()->type() == Cantor::HelpResult::Type)
+    if (expr->results().last()->type() == Cantor::HelpResult::Type)
         return; // Help is handled elsewhere
 
-    if (expr->result()->type() == Cantor::TextResult::Type &&
-        expr->result()->toHtml().trimmed().isEmpty()) {
-        return;
-    } else if (!m_resultItem) {
-        m_resultItem = ResultItem::create(this, expr->result());
-        qDebug() << "new result";
-        animateSizeChange();
-    } else {
-        m_resultItem = m_resultItem->updateFromResult(expr->result());
-        qDebug() << "update result";
+    //CommandEntry::updateEntry() is only called if the worksheet view is resized
+    //or when we got a new result(s).
+    //In the second case the number of results and the number of result graphic objects is different
+    //and we add a new graphic objects for the new result(s) (new result(s) are located in the end).
+    if (m_resultItems.size() < expr->results().size())
+    {
+        for (int i = m_resultItems.size(); i < expr->results().size(); i++)
+            m_resultItems << ResultItem::create(this, expr->results()[i]);
         animateSizeChange();
     }
 }
 
 void CommandEntry::expressionChangedStatus(Cantor::Expression::Status status)
 {
-    QString text;
     switch (status)
     {
-    case Cantor::Expression::Error:
-        text = m_expression->errorMessage();
+    case Cantor::Expression::Computing:
+    {
+        //change the background of the promt item and start animating it (fade in/out).
+        //don't start the animation immediately in order to avoid unwanted flickering for "short" commands,
+        //start the animation after 1 second passed.
+        if (worksheet()->animationsEnabled())
+        {
+            const int id = m_expression->id();
+            QTimer::singleShot(1000, this, [this, id] () {
+                if(m_expression->status() == Cantor::Expression::Computing && m_expression->id() == id)
+                    m_promptItemAnimation->start();
+            });
+        }
         break;
+    }
+    case Cantor::Expression::Error:
     case Cantor::Expression::Interrupted:
-        text = i18n("Interrupted");
+        m_promptItemAnimation->stop();
+        m_promptItem->setOpacity(1.);
+
+        m_commandItem->setFocusAt(WorksheetTextItem::BottomRight, 0);
+
+        if(!m_errorItem)
+        {
+            m_errorItem = new WorksheetTextItem(this, Qt::TextSelectableByMouse);
+        }
+
+        if (status == Cantor::Expression::Error)
+            m_errorItem->setHtml(m_expression->errorMessage());
+        else
+            m_errorItem->setHtml(i18n("Interrupted"));
+
+        recalculateSize();
         break;
     case Cantor::Expression::Done:
+        m_promptItemAnimation->stop();
+        m_promptItem->setOpacity(1.);
         evaluateNext(m_evaluationOption);
         m_evaluationOption = DoNothing;
-        return;
+        break;
     default:
-        return;
+        break;
     }
+}
 
-    m_commandItem->setFocusAt(WorksheetTextItem::BottomRight, 0);
-
-    if(!m_errorItem)
-    {
-        m_errorItem = new WorksheetTextItem(this, Qt::TextSelectableByMouse);
-    }
-
-    m_errorItem->setHtml(text);
-    recalculateSize();
+void CommandEntry::animatePromptItem() {
+    if(m_expression->status() == Cantor::Expression::Computing)
+        m_promptItemAnimation->start();
 }
 
 bool CommandEntry::isEmpty()
 {
     if (m_commandItem->toPlainText().trimmed().isEmpty()) {
-        if (m_resultItem)
+        if (!m_resultItems.isEmpty())
             return false;
         return true;
     }
@@ -570,7 +870,22 @@ void CommandEntry::showAdditionalInformationPrompt(const QString& question)
 {
     WorksheetTextItem* questionItem = new WorksheetTextItem(this, Qt::TextSelectableByMouse);
     WorksheetTextItem* answerItem = new WorksheetTextItem(this, Qt::TextEditorInteraction);
+
+    //change the color and the font for when asking for additional information in order to
+    //better discriminate from the usual input in the command entry
+    KColorScheme scheme = KColorScheme(QPalette::Normal, KColorScheme::View);
+    QColor color = scheme.foreground(KColorScheme::PositiveText).color();
+
+    QFont font;
+    font.setItalic(true);
+
+    questionItem->setFont(font);
+    questionItem->setDefaultTextColor(color);
+    answerItem->setFont(font);
+    answerItem->setDefaultTextColor(color);
+
     questionItem->setPlainText(question);
+
     m_informationItems.append(questionItem);
     m_informationItems.append(answerItem);
     connect(answerItem, &WorksheetTextItem::moveToPrevious, this, &CommandEntry::moveToPreviousItem);
@@ -578,21 +893,46 @@ void CommandEntry::showAdditionalInformationPrompt(const QString& question)
 
     connect(answerItem, &WorksheetTextItem::execute, this, &CommandEntry::addInformation);
     answerItem->setFocus();
+
     recalculateSize();
 }
 
-void CommandEntry::removeResult()
+void CommandEntry::removeResults()
 {
+    //clear the Result objects
     if(m_expression)
-    {
-        m_expression->clearResult();
-    }
+        m_expression->clearResults();
+}
 
-    if (m_resultItem) {
-        QGraphicsObject* obj = m_resultItem->graphicsObject();
-        m_resultItem = nullptr;
-        fadeOutItem(obj);
-    }
+void CommandEntry::removeResult(Cantor::Result* result)
+{
+    if (m_expression)
+        m_expression->removeResult(result);
+}
+
+void CommandEntry::removeResultItem(int index)
+{
+    fadeOutItem(m_resultItems[index]->graphicsObject());
+    m_resultItems.remove(index);
+    recalculateSize();
+}
+
+void CommandEntry::clearResultItems()
+{
+    //fade out all result graphic objects
+    for(auto* item : m_resultItems)
+        fadeOutItem(item->graphicsObject());
+
+    m_resultItems.clear();
+    recalculateSize();
+}
+
+void CommandEntry::replaceResultItem(int index)
+{
+    ResultItem* previousItem = m_resultItems[index];
+    m_resultItems[index] = ResultItem::create(this, m_expression->results()[index]);
+    previousItem->deleteLater();
+    recalculateSize();
 }
 
 void CommandEntry::removeContextHelp()
@@ -604,7 +944,6 @@ void CommandEntry::removeContextHelp()
         m_completionBox->hide();
 }
 
-
 void CommandEntry::updatePrompt()
 {
     KColorScheme color = KColorScheme( QPalette::Normal, KColorScheme::View);
@@ -627,6 +966,8 @@ void CommandEntry::updatePrompt()
     {
         if(m_expression ->status() == Cantor::Expression::Computing&&worksheet()->isRunning())
             cformat.setForeground(color.foreground(KColorScheme::PositiveText));
+        else if(m_expression ->status() == Cantor::Expression::Queued)
+            cformat.setForeground(color.foreground(KColorScheme::InactiveText));
         else if(m_expression ->status() == Cantor::Expression::Error)
             cformat.setForeground(color.foreground(KColorScheme::NegativeText));
         else if(m_expression ->status() == Cantor::Expression::Interrupted)
@@ -720,12 +1061,15 @@ WorksheetCursor CommandEntry::search(const QString& pattern, unsigned flags,
     if (p.textItem() == m_errorItem)
         p = WorksheetCursor();
 
-    WorksheetTextItem* textResult = dynamic_cast<WorksheetTextItem*>
-        (m_resultItem);
-    if (textResult && flags & WorksheetEntry::SearchResult) {
-        cursor = textResult->search(pattern, qt_flags, p);
-        if (!cursor.isNull())
-            return WorksheetCursor(this, textResult, cursor);
+    for (auto* resultItem : m_resultItems)
+    {
+        WorksheetTextItem* textResult = dynamic_cast<WorksheetTextItem*>
+            (resultItem);
+        if (textResult && flags & WorksheetEntry::SearchResult) {
+            cursor = textResult->search(pattern, qt_flags, p);
+            if (!cursor.isNull())
+                return WorksheetCursor(this, textResult, cursor);
+        }
     }
 
     return WorksheetCursor();
@@ -757,10 +1101,13 @@ void CommandEntry::layOutForWidth(qreal w, bool force)
         width = qMax(width, m_errorItem->width());
     }
 
-    if (m_resultItem) {
+    for (auto* resultItem : m_resultItems)
+    {
+        if (!resultItem)
+            continue;
         y += VerticalSpacing;
-        y += m_resultItem->setGeometry(x, y, w-x);
-        width = qMax(width, m_resultItem->width());
+        y += resultItem->setGeometry(x, y, w-x);
+        width = qMax(width, resultItem->width());
     }
     y += VerticalMargin;
 
