@@ -25,7 +25,7 @@
 #include <QObject>
 #include <KPluginMetaData>
 
-#include "cantor_export.h"
+#include <cantor_export.h>
 
 namespace Cantor
 {
@@ -45,7 +45,7 @@ class CANTOR_EXPORT Assistant : public QObject, public KXMLGUIClient
      * Create a new assistant
      * @param parent the parent Object @see QObject
      **/
-    Assistant( QObject* parent );
+    explicit Assistant( QObject* parent );
     /**
      * Destructor
      */
@@ -59,7 +59,7 @@ class CANTOR_EXPORT Assistant : public QObject, public KXMLGUIClient
 
     /**
      * Sets the properties of this Assistant
-     * accodring to KPluginMetaData
+     * according to KPluginMetaData
      * @param info KPluginMetaData
      */
     void setPluginInfo(KPluginMetaData info);
