@@ -23,7 +23,7 @@
 
 #include <KCompletion>
 
-#include "cantor_export.h"
+#include <cantor_export.h>
 
 namespace Cantor
 {
@@ -31,7 +31,7 @@ class CompletionObjectPrivate;
 class Session;
 
 /**
- * This Object is used to provide a Tab Completion, in an asynchroneous way.
+ * This Object is used to provide a Tab Completion, in an asynchronous way.
  * Each backend, supporting tab completion, needs to provide their own
  * CompletionObject, that reimplements the fetching of the completions
  * and emits done() as soon as the completions are available
@@ -46,7 +46,7 @@ class CANTOR_EXPORT CompletionObject : public KCompletion
      * Constructor
      * @param parent the session, this object belongs to
      */
-    CompletionObject(Session* parent);
+    explicit CompletionObject(Session* parent);
     ///Destructor
     ~CompletionObject() override;
 
@@ -81,7 +81,7 @@ class CANTOR_EXPORT CompletionObject : public KCompletion
      * This triggers an asynchronous fetching of completions,
      * which emits done() when done.
      * @param line the line that is to be completed
-     * @param index the cursor postition in line
+     * @param index the cursor position in line
      */
     void setLine(const QString& line, int index);
     /**
@@ -98,7 +98,8 @@ class CANTOR_EXPORT CompletionObject : public KCompletion
      * FinalCompletion some postprocessing is done asynchronously.
      * Emits lineDone when finished.
      * @param comp the completion that's to be processed
-     * @param type whether the completion is final
+     * @param mode the mode of completion. Can be PreliminaryCompletion
+     * (insert only) or FinalCompletion (also add () for functions, etc.)
      */
     void completeLine(const QString& comp, LineCompletionMode mode);
 
@@ -168,7 +169,7 @@ class CANTOR_EXPORT CompletionObject : public KCompletion
   protected Q_SLOTS:
     /**
      * This function should be reimplemented to start the actual fetching
-     * of the completions. It can be asynchroneous.
+     * of the completions. It can be asynchronous.
      * Remember to emit fetchingDone, if the fetching is complete
      */
     virtual void fetchCompletions() = 0;
