@@ -22,7 +22,7 @@
 #define _EPSRESULT_H
 
 #include "result.h"
-#include "cantor_export.h"
+#include <cantor_export.h>
 #include <QUrl>
 
 namespace Cantor
@@ -33,21 +33,21 @@ class CANTOR_EXPORT EpsResult : public Result
 {
   public:
     enum {Type=5};
-    EpsResult( const QUrl& url);
+    explicit EpsResult( const QUrl& url);
     ~EpsResult() override;
 
-    QString toHtml() Q_DECL_OVERRIDE;
-    QString toLatex() Q_DECL_OVERRIDE;
-    QVariant data() Q_DECL_OVERRIDE;
-    QUrl url() Q_DECL_OVERRIDE;
+    QString toHtml() override;
+    QString toLatex() override;
+    QVariant data() override;
+    QUrl url() override;
 
-    int type() Q_DECL_OVERRIDE;
-    QString mimeType() Q_DECL_OVERRIDE;
+    int type() override;
+    QString mimeType() override;
 
-    QDomElement toXml(QDomDocument& doc) Q_DECL_OVERRIDE;
-    void saveAdditionalData(KZip* archive) Q_DECL_OVERRIDE;
+    QDomElement toXml(QDomDocument& doc) override;
+    void saveAdditionalData(KZip* archive) override;
 
-    void save(const QString& filename) Q_DECL_OVERRIDE;
+    void save(const QString& filename) override;
 
   private:
     EpsResultPrivate* d;
