@@ -34,12 +34,12 @@ class CANTOR_EXPORT HelpResult : public TextResult
 {
   public:
     enum {Type=3};
-    HelpResult( const QString& text);
-    ~HelpResult() override;
+    explicit HelpResult( const QString& text);
+    ~HelpResult() override = default;
 
-    int type() Q_DECL_OVERRIDE;
+    int type() override;
 
-    QDomElement toXml(QDomDocument& doc) Q_DECL_OVERRIDE;
+    QDomElement toXml(QDomDocument& doc) override;
 };
 
 }
