@@ -23,10 +23,13 @@ using namespace Cantor;
 
 #include "backend.h"
 
+#include <QTimer>
+#include <QQueue>
+
 class Cantor::SessionPrivate
 {
   public:
-    SessionPrivate() : backend(nullptr), status(Session::Done), typesettingEnabled(false), expressionCount(0)
+    SessionPrivate() : backend(nullptr), status(Session::Disable), typesettingEnabled(false), expressionCount(0)
     {
     }
 
@@ -34,6 +37,7 @@ class Cantor::SessionPrivate
     Session::Status status;
     bool typesettingEnabled;
     int expressionCount;
+    QList<Cantor::Expression*> expressionQueue;
 };
 
 Session::Session( Backend* backend ) : QObject(backend), d(new SessionPrivate)
@@ -46,9 +50,28 @@ Session::~Session()
     delete d;
 }
 
-Expression* Session::evaluateExpression(const QString& command)
+QList<Expression*>& Cantor::Session::expressionQueue() const
 {
-    return evaluateExpression(command, Expression::DoNotDelete);
+    return d->expressionQueue;
+}
+
+void Session::enqueueExpression(Expression* expr)
+{
+    d->expressionQueue.append(expr);
+
+    //run the newly added expression immediately if it's the only one in the queue
+    if (d->expressionQueue.size() == 1)
+    {
+     changeStatus(Cantor::Session::Running);
+     runFirstExpression();
+    }
+    else
+        expr->setStatus(Cantor::Expression::Queued);
+}
+
+void Session::runFirstExpression()
+{
+
 }
 
 Backend* Session::backend()
@@ -122,6 +145,3 @@ int Session::nextExpressionId()
 {
     return d->expressionCount++;
 }
-
-
-
