@@ -22,7 +22,7 @@
 #define _SYNTAXHELPOBJECT_H
 
 #include <QObject>
-#include "cantor_export.h"
+#include <cantor_export.h>
 
 namespace Cantor{
 
@@ -30,8 +30,8 @@ class SyntaxHelpObjectPrivate;
 class Session;
 
 /**
- * Object, used to display Syntax informtion to a given command
- * It is designed for asynchroneous use. The Object emits done
+ * Object, used to display Syntax information to a given command
+ * It is designed for asynchronous use. The Object emits done
  * as soon as fetching of the Help is finished
  *
  * @author Alexander Rieder
@@ -49,7 +49,7 @@ class CANTOR_EXPORT SyntaxHelpObject : public QObject
     /**
      * Destructor
      */
-    ~SyntaxHelpObject();
+    ~SyntaxHelpObject() override;
 
     /**
      * Start fetching the syntax help, emitting done() when done.
