@@ -59,7 +59,7 @@ TextResult::~TextResult()
 
 QString TextResult::toHtml()
 {
-    QString s=d->data;
+    QString s=d->data.toHtmlEscaped();
     s.replace(QLatin1Char('\n'), QLatin1String("<br/>\n"));
     return s;
 }
@@ -122,6 +122,3 @@ void TextResult::save(const QString& filename)
 
     file.close();
 }
-
-
-
