@@ -21,7 +21,7 @@
 #ifndef _WORKSHEET_ACCESS_INTERFACE_H
 #define _WORKSHEET_ACCESS_INTERFACE_H
 
-#include "cantor_export.h"
+#include <cantor_export.h>
 
 #include<QObject>
 
@@ -29,14 +29,14 @@
 namespace Cantor
 {
     class Session;
-    
+
 class CANTOR_EXPORT WorksheetAccessInterface : public QObject
 {
   Q_OBJECT
-  public:  
+  public:
     static QLatin1String Name;
-    WorksheetAccessInterface(QObject* parent);
-    ~WorksheetAccessInterface() override;
+    explicit WorksheetAccessInterface(QObject* parent);
+    ~WorksheetAccessInterface() override = default;
   public:
     virtual QByteArray saveWorksheetToByteArray() = 0;
     virtual void loadWorksheetFromByteArray(QByteArray* data) = 0;
@@ -47,7 +47,7 @@ class CANTOR_EXPORT WorksheetAccessInterface : public QObject
       virtual void interrupt() = 0;
 
   Q_SIGNALS:
-      void sessionChanged();
+      void modified();
 };
 
 }
