@@ -19,13 +19,15 @@
     Copyright (C) 2012 Martin Kuettler <martin.kuettler@gmail.com>
  */
 
+#include "worksheet.h"
+
 #include <QBuffer>
 #include <QDebug>
 #include <QDrag>
 #include <QGraphicsWidget>
 #include <QPrinter>
 #include <QTimer>
-#include <QtXmlPatterns/QXmlQuery>
+#include <QXmlQuery>
 
 #include <KMessageBox>
 #include <KActionCollection>
@@ -33,10 +35,10 @@
 #include <KFontSizeAction>
 #include <KToggleAction>
 
-#include "worksheet.h"
 #include "settings.h"
 #include "commandentry.h"
 #include "textentry.h"
+#include "markdownentry.h"
 #include "latexentry.h"
 #include "imageentry.h"
 #include "pagebreakentry.h"
@@ -51,7 +53,9 @@
 
 const double Worksheet::LeftMargin = 4;
 const double Worksheet::RightMargin = 4;
-const double Worksheet::TopMargin = 4;
+const double Worksheet::TopMargin = 12;
+const double Worksheet::EntryCursorLength = 30;
+const double Worksheet::EntryCursorWidth = 2;
 
 Worksheet::Worksheet(Cantor::Backend* backend, QWidget* parent)
     : QGraphicsScene(parent)
@@ -68,8 +72,23 @@ Worksheet::Worksheet(Cantor::Backend* backend, QWidget* parent)
     m_protrusion = 0;
     m_dragScrollTimer = nullptr;
 
+    m_choosenCursorEntry = nullptr;
+    m_isCursorEntryAfterLastEntry = false;
+
+    m_entryCursorItem = addLine(0,0,0,0);
+    const QColor& color = (palette().color(QPalette::Base).lightness() < 128) ? Qt::white : Qt::black;
+    QPen pen(color);
+    pen.setWidth(EntryCursorWidth);
+    m_entryCursorItem->setPen(pen);
+    m_entryCursorItem->hide();
+
+    m_cursorItemTimer = new QTimer(this);
+    connect(m_cursorItemTimer, &QTimer::timeout, this, &Worksheet::animateEntryCursor);
+    m_cursorItemTimer->start(500);
+
     m_isPrinting = false;
     m_loginDone = false;
+    m_readOnly = false;
     m_isLoadingFromFile = false;
 
     enableHighlighting(Settings::self()->highlightDefault());
@@ -114,7 +133,7 @@ void Worksheet::print(QPrinter* printer)
     painter.scale(scale, scale);
     painter.setRenderHint(QPainter::Antialiasing);
     WorksheetEntry* entry = firstEntry();
-    qreal y = 0;
+    qreal y = TopMargin;
 
     while (entry) {
         qreal h = 0;
@@ -180,6 +199,7 @@ void Worksheet::updateLayout()
         makeVisible(worksheetCursor());
     else if (atEnd)
         worksheetView()->scrollToEnd();
+    drawEntryCursor();
 }
 
 void Worksheet::updateEntrySize(WorksheetEntry* entry)
@@ -201,6 +221,7 @@ void Worksheet::updateEntrySize(WorksheetEntry* entry)
         makeVisible(worksheetCursor());
     else if (atEnd)
         worksheetView()->scrollToEnd();
+    drawEntryCursor();
 }
 
 void Worksheet::addProtrusion(qreal width)
@@ -228,7 +249,7 @@ void Worksheet::removeProtrusion(qreal width)
         m_itemProtrusions.remove(width);
         if (width == m_protrusion) {
             qreal max = -1;
-            foreach (qreal p, m_itemProtrusions.keys()) {
+            for (qreal p : m_itemProtrusions.keys()) {
                 if (p > max)
                     max = p;
             }
@@ -309,6 +330,10 @@ void Worksheet::setWorksheetCursor(const WorksheetCursor& cursor)
 WorksheetEntry* Worksheet::currentEntry()
 {
     QGraphicsItem* item = focusItem();
+    // Entry cursor activate
+    if (m_choosenCursorEntry || m_isCursorEntryAfterLastEntry)
+        return nullptr;
+
     if (!item /*&& !hasFocus()*/)
         item = m_lastFocusedTextItem;
     /*else
@@ -401,6 +426,10 @@ void Worksheet::focusEntry(WorksheetEntry *entry)
 
 void Worksheet::startDrag(WorksheetEntry* entry, QDrag* drag)
 {
+    if (m_readOnly)
+        return;
+
+    resetEntryCursor();
     m_dragEntry = entry;
     WorksheetEntry* prev = entry->previous();
     WorksheetEntry* next = entry->next();
@@ -453,7 +482,7 @@ void Worksheet::startDrag(WorksheetEntry* entry, QDrag* drag)
 void Worksheet::evaluate()
 {
     qDebug()<<"evaluate worksheet";
-    if (!m_loginDone)
+    if (!m_loginDone && !m_readOnly)
         loginToSession();
 
     firstEntry()->evaluate(WorksheetEntry::EvaluateNext);
@@ -463,7 +492,7 @@ void Worksheet::evaluate()
 
 void Worksheet::evaluateCurrentEntry()
 {
-    if (!m_loginDone)
+    if (!m_loginDone && !m_readOnly)
         loginToSession();
 
     WorksheetEntry* entry = currentEntry();
@@ -479,18 +508,6 @@ bool Worksheet::completionEnabled()
 
 void Worksheet::showCompletion()
 {
-    //for backends which fetch the completion from the corresponding interpreter
-    //we need to do the login first.
-    //TODO: ideally, this logic is implemented in the actual session classes
-    //and the worksheet doesn't need to know anything about this.
-    //But as long as we have some additional logic in Worksheet::loginToSession(),
-    //we need to handle this here.
-    if (!m_loginDone && (m_session->backend()->name() == QLatin1String("Julia")
-                    || m_session->backend()->name() == QLatin1String("Lua")
-                    || m_session->backend()->name() == QLatin1String("R")
-                    || m_session->backend()->name() == QLatin1String("Octave")) )
-        loginToSession();
-
     WorksheetEntry* current = currentEntry();
     current->showCompletion();
 }
@@ -511,6 +528,7 @@ WorksheetEntry* Worksheet::appendEntry(const int type)
         updateLayout();
         makeVisible(entry);
         focusEntry(entry);
+        emit modified();
     }
     return entry;
 }
@@ -525,6 +543,10 @@ WorksheetEntry* Worksheet::appendTextEntry()
    return appendEntry(TextEntry::Type);
 }
 
+WorksheetEntry* Worksheet::appendMarkdownEntry()
+{
+   return appendEntry(MarkdownEntry::Type);
+}
 
 WorksheetEntry* Worksheet::appendPageBreakEntry()
 {
@@ -579,6 +601,7 @@ WorksheetEntry* Worksheet::insertEntry(const int type, WorksheetEntry* current)
         else
             setLastEntry(entry);
         updateLayout();
+        emit modified();
     } else {
         entry = next;
     }
@@ -593,6 +616,11 @@ WorksheetEntry* Worksheet::insertTextEntry(WorksheetEntry* current)
     return insertEntry(TextEntry::Type, current);
 }
 
+WorksheetEntry* Worksheet::insertMarkdownEntry(WorksheetEntry* current)
+{
+    return insertEntry(MarkdownEntry::Type, current);
+}
+
 WorksheetEntry* Worksheet::insertCommandEntry(WorksheetEntry* current)
 {
     return insertEntry(CommandEntry::Type, current);
@@ -645,6 +673,7 @@ WorksheetEntry* Worksheet::insertEntryBefore(int type, WorksheetEntry* current)
         else
             setFirstEntry(entry);
         updateLayout();
+        emit modified();
     }
 
     focusEntry(entry);
@@ -656,6 +685,11 @@ WorksheetEntry* Worksheet::insertTextEntryBefore(WorksheetEntry* current)
     return insertEntryBefore(TextEntry::Type, current);
 }
 
+WorksheetEntry* Worksheet::insertMarkdownEntryBefore(WorksheetEntry* current)
+{
+    return insertEntryBefore(MarkdownEntry::Type, current);
+}
+
 WorksheetEntry* Worksheet::insertCommandEntryBefore(WorksheetEntry* current)
 {
     return insertEntryBefore(CommandEntry::Type, current);
@@ -756,11 +790,14 @@ void Worksheet::rehighlight()
             WorksheetTextItem* item = entry->highlightItem();
             if (!item)
                 continue;
+            QTextCursor cursor(item->document());
+            cursor.beginEditBlock();
             for (QTextBlock b = item->document()->firstBlock();
                  b.isValid(); b = b.next())
             {
                 b.layout()->clearAdditionalFormats();
             }
+            cursor.endEditBlock();
         }
         update();
     }
@@ -773,12 +810,11 @@ void Worksheet::enableHighlighting(bool highlight)
         if(m_highlighter)
             m_highlighter->deleteLater();
 
-        //octave highligther fetches the function names from octave -> we need to login here
-        //TODO: re-design this later - highlighting should also be possible in the absence of the actual backend
-        if (!m_loginDone && m_session->backend()->name() == QLatin1String("Octave"))
-            loginToSession();
+        if (!m_readOnly)
+            m_highlighter=session()->syntaxHighlighter(this);
+        else
+            m_highlighter=nullptr;
 
-        m_highlighter=session()->syntaxHighlighter(this);
         if(!m_highlighter)
             m_highlighter=new Cantor::DefaultHighlighter(this);
 
@@ -806,7 +842,12 @@ Cantor::Session* Worksheet::session()
 
 bool Worksheet::isRunning()
 {
-    return m_session->status()==Cantor::Session::Running;
+    return m_session && m_session->status()==Cantor::Session::Running;
+}
+
+bool Worksheet::isReadOnly()
+{
+    return m_readOnly;
 }
 
 bool Worksheet::showExpressionIds()
@@ -834,7 +875,7 @@ QDomDocument Worksheet::toXML(KZip* archive)
 {
     QDomDocument doc( QLatin1String("CantorWorksheet") );
     QDomElement root=doc.createElement( QLatin1String("Worksheet") );
-    root.setAttribute(QLatin1String("backend"), m_session->backend()->name());
+    root.setAttribute(QLatin1String("backend"), (m_session ? m_session->backend()->name(): m_backendName));
     doc.appendChild(root);
 
     for( WorksheetEntry* entry = firstEntry(); entry; entry = entry->next())
@@ -902,14 +943,19 @@ void Worksheet::savePlain(const QString& filename)
     QString commentStartingSeq = QLatin1String("");
     QString commentEndingSeq = QLatin1String("");
 
-    Cantor::Backend * const backend=session()->backend();
-    if (backend->extensions().contains(QLatin1String("ScriptExtension")))
+    if (!m_readOnly)
     {
-        Cantor::ScriptExtension* e=dynamic_cast<Cantor::ScriptExtension*>(backend->extension(QLatin1String(("ScriptExtension"))));
-        cmdSep=e->commandSeparator();
-        commentStartingSeq = e->commentStartingSequence();
-        commentEndingSeq = e->commentEndingSequence();
+        Cantor::Backend * const backend=session()->backend();
+        if (backend->extensions().contains(QLatin1String("ScriptExtension")))
+        {
+            Cantor::ScriptExtension* e=dynamic_cast<Cantor::ScriptExtension*>(backend->extension(QLatin1String(("ScriptExtension"))));
+            cmdSep=e->commandSeparator();
+            commentStartingSeq = e->commentStartingSequence();
+            commentEndingSeq = e->commentEndingSequence();
+        }
     }
+    else
+        KMessageBox::information(worksheetView(), i18n("In read-only mode Cantor couldn't guarantee, that the export will be valid for %1", m_backendName), i18n("Cantor"));
 
     QTextStream stream(&file);
 
@@ -965,7 +1011,7 @@ bool Worksheet::load(const QString& filename )
     }
 
     bool rc = load(&file);
-    if (rc)
+    if (rc && !m_readOnly)
         m_session->setWorksheetPath(filename);
 
     return rc;
@@ -1006,22 +1052,31 @@ bool Worksheet::load(QIODevice* device)
     QDomElement root=doc.documentElement();
 //     qDebug()<<root.tagName();
 
-    const QString backendName=root.attribute(QLatin1String("backend"));
-    Cantor::Backend* b=Cantor::Backend::createBackend(backendName);
+    m_backendName=root.attribute(QLatin1String("backend"));
+    Cantor::Backend* b=Cantor::Backend::getBackend(m_backendName);
     if (!b)
     {
         QApplication::restoreOverrideCursor();
-        KMessageBox::error(worksheetView(), i18n("The backend with which this file was generated is not installed. It needs %1", backendName), i18n("Cantor"));
-        return false;
+        KMessageBox::information(worksheetView(), i18n("%1 backend was not found. Editing and executing entries is not possible", m_backendName), i18n("Cantor"));
+        m_readOnly = true;
     }
+    else
+        m_readOnly = false;
 
-    if(!b->isEnabled())
+    if(!m_readOnly && !b->isEnabled())
     {
         QApplication::restoreOverrideCursor();
         KMessageBox::information(worksheetView(), i18n("There are some problems with the %1 backend,\n"\
                                             "please check your configuration or install the needed packages.\n"
-                                            "You will only be able to view this worksheet.", backendName), i18n("Cantor"));
+                                            "You will only be able to view this worksheet.", m_backendName), i18n("Cantor"));
+        m_readOnly = true;
+    }
 
+    if (m_readOnly)
+    {
+        // TODO: Handle this here?
+        for (QAction* action : m_richTextActionList)
+            action->setEnabled(false);
     }
 
     m_isLoadingFromFile = true;
@@ -1034,14 +1089,14 @@ bool Worksheet::load(QIODevice* device)
     //file can only be loaded in a worksheet that was not eidted/modified yet (s.a. CantorShell::load())
     //in this case on the default "first entry" is available -> delete it.
     if (m_firstEntry) {
-        delete m_firstEntry;
+        m_firstEntry->deleteLater();
         m_firstEntry = nullptr;
     }
 
-    //delete all items from the scene
-    clear();
+    resetEntryCursor();
 
-    m_session=b->createSession();
+    if (!m_readOnly)
+        m_session=b->createSession();
 
     qDebug()<<"loading entries";
     QDomElement expressionChild = root.firstChildElement();
@@ -1056,6 +1111,10 @@ bool Worksheet::load(QIODevice* device)
         {
             entry = appendTextEntry();
             entry->setContent(expressionChild, file);
+        } else if (tag == QLatin1String("Markdown"))
+        {
+            entry = appendMarkdownEntry();
+            entry->setContent(expressionChild, file);
         } else if (tag == QLatin1String("Latex"))
         {
             entry = appendLatexEntry();
@@ -1071,16 +1130,25 @@ bool Worksheet::load(QIODevice* device)
             entry->setContent(expressionChild, file);
         }
 
+        if (m_readOnly && entry)
+        {
+            entry->setAcceptHoverEvents(false);
+            entry = nullptr;
+        }
+
         expressionChild = expressionChild.nextSiblingElement();
     }
 
+    if (m_readOnly)
+        clearFocus();
+
     m_isLoadingFromFile = false;
 
     //Set the Highlighting, depending on the current state
     //If the session isn't logged in, use the default
     enableHighlighting( m_highlighter!=nullptr || Settings::highlightDefault() );
 
-    emit sessionChanged();
+    emit loaded();
     return true;
 }
 
@@ -1091,15 +1159,22 @@ void Worksheet::gotResult(Cantor::Expression* expr)
 
     if(expr==nullptr)
         return;
-    //We're only interested in help results, others are handled by the WorksheetEntry
-    if(expr->result()&&expr->result()->type()==Cantor::HelpResult::Type)
-    {
-        QString help=expr->result()->toHtml();
-        //Do some basic LaTeX replacing
-        help.replace(QRegExp(QLatin1String("\\\\code\\{([^\\}]*)\\}")), QLatin1String("<b>\\1</b>"));
-        help.replace(QRegExp(QLatin1String("\\$([^\\$])\\$")), QLatin1String("<i>\\1</i>"));
 
-        emit showHelp(help);
+    //We're only interested in help results, others are handled by the WorksheetEntry
+    for (auto* result : expr->results())
+    {
+        if(result && result->type()==Cantor::HelpResult::Type)
+        {
+            QString help = result->toHtml();
+            //Do some basic LaTeX replacing
+            help.replace(QRegExp(QLatin1String("\\\\code\\{([^\\}]*)\\}")), QLatin1String("<b>\\1</b>"));
+            help.replace(QRegExp(QLatin1String("\\$([^\\$])\\$")), QLatin1String("<i>\\1</i>"));
+
+            emit showHelp(help);
+
+            //TODO: break after the first help result found, not clear yet how to handle multiple requests for help within one single command (e.g. ??ev;??int).
+            break;
+        }
     }
 }
 
@@ -1151,33 +1226,53 @@ void Worksheet::populateMenu(QMenu *menu, QPointF pos)
         QMenu* insert = new QMenu(menu);
         QMenu* insertBefore = new QMenu(menu);
 
-        insert->addAction(i18n("Command Entry"), entry, SLOT(insertCommandEntry()));
-        insert->addAction(i18n("Text Entry"), entry, SLOT(insertTextEntry()));
-        insert->addAction(i18n("LaTeX Entry"), entry, SLOT(insertLatexEntry()));
-        insert->addAction(i18n("Image"), entry, SLOT(insertImageEntry()));
-        insert->addAction(i18n("Page Break"), entry, SLOT(insertPageBreakEntry()));
+        insert->addAction(QIcon::fromTheme(QLatin1String("run-build")), i18n("Command Entry"), entry, SLOT(insertCommandEntry()));
+        insert->addAction(QIcon::fromTheme(QLatin1String("draw-text")), i18n("Text Entry"), entry, SLOT(insertTextEntry()));
+#ifdef Discount_FOUND
+        insert->addAction(QIcon::fromTheme(QLatin1String("text-x-markdown")), i18n("Markdown Entry"), entry, SLOT(insertMarkdownEntry()));
+#endif
+#ifdef WITH_EPS
+        insert->addAction(QIcon::fromTheme(QLatin1String("text-x-tex")), i18n("LaTeX Entry"), entry, SLOT(insertLatexEntry()));
+#endif
+        insert->addAction(QIcon::fromTheme(QLatin1String("image-x-generic")), i18n("Image"), entry, SLOT(insertImageEntry()));
+        insert->addAction(QIcon::fromTheme(QLatin1String("go-next-view-page")), i18n("Page Break"), entry, SLOT(insertPageBreakEntry()));
 
-        insertBefore->addAction(i18n("Command Entry"), entry, SLOT(insertCommandEntryBefore()));
-        insertBefore->addAction(i18n("Text Entry"), entry, SLOT(insertTextEntryBefore()));
-        insertBefore->addAction(i18n("LaTeX Entry"), entry, SLOT(insertLatexEntryBefore()));
-        insertBefore->addAction(i18n("Image"), entry, SLOT(insertImageEntryBefore()));
-        insertBefore->addAction(i18n("Page Break"), entry, SLOT(insertPageBreakEntryBefore()));
+        insertBefore->addAction(QIcon::fromTheme(QLatin1String("run-build")), i18n("Command Entry"), entry, SLOT(insertCommandEntryBefore()));
+        insertBefore->addAction(QIcon::fromTheme(QLatin1String("draw-text")), i18n("Text Entry"), entry, SLOT(insertTextEntryBefore()));
+#ifdef Discount_FOUND
+        insertBefore->addAction(QIcon::fromTheme(QLatin1String("text-x-markdown")), i18n("Markdown Entry"), entry, SLOT(insertMarkdownEntryBefore()));
+#endif
+#ifdef WITH_EPS
+        insertBefore->addAction(QIcon::fromTheme(QLatin1String("text-x-tex")), i18n("LaTeX Entry"), entry, SLOT(insertLatexEntryBefore()));
+#endif
+        insertBefore->addAction(QIcon::fromTheme(QLatin1String("image-x-generic")), i18n("Image"), entry, SLOT(insertImageEntryBefore()));
+        insertBefore->addAction(QIcon::fromTheme(QLatin1String("go-next-view-page")), i18n("Page Break"), entry, SLOT(insertPageBreakEntryBefore()));
 
-        insert->setTitle(i18n("Insert"));
-        insertBefore->setTitle(i18n("Insert Before"));
+        insert->setTitle(i18n("Insert Entry After"));
+        insert->setIcon(QIcon::fromTheme(QLatin1String("edit-table-insert-row-below")));
+        insertBefore->setTitle(i18n("Insert Entry Before"));
+        insertBefore->setIcon(QIcon::fromTheme(QLatin1String("edit-table-insert-row-above")));
         menu->addMenu(insert);
         menu->addMenu(insertBefore);
     } else {
-        menu->addAction(i18n("Insert Command Entry"), this, SLOT(appendCommandEntry()));
-        menu->addAction(i18n("Insert Text Entry"), this, SLOT(appendTextEntry()));
-        menu->addAction(i18n("Insert LaTeX Entry"), this, SLOT(appendLatexEntry()));
-        menu->addAction(i18n("Insert Image"), this, SLOT(appendImageEntry()));
-        menu->addAction(i18n("Insert Page Break"), this, SLOT(appendPageBreakEntry()));
+        menu->addAction(QIcon::fromTheme(QLatin1String("run-build")), i18n("Insert Command Entry"), this, SLOT(appendCommandEntry()));
+        menu->addAction(QIcon::fromTheme(QLatin1String("draw-text")), i18n("Insert Text Entry"), this, SLOT(appendTextEntry()));
+#ifdef Discount_FOUND
+        menu->addAction(QIcon::fromTheme(QLatin1String("text-x-markdown")), i18n("Insert Markdown Entry"), this, SLOT(appendMarkdownEntry()));
+#endif
+#ifdef WITH_EPS
+        menu->addAction(QIcon::fromTheme(QLatin1String("text-x-tex")), i18n("Insert LaTeX Entry"), this, SLOT(appendLatexEntry()));
+#endif
+        menu->addAction(QIcon::fromTheme(QLatin1String("image-x-generic")), i18n("Insert Image"), this, SLOT(appendImageEntry()));
+        menu->addAction(QIcon::fromTheme(QLatin1String("go-next-view-page")), i18n("Insert Page Break"), this, SLOT(appendPageBreakEntry()));
     }
 }
 
 void Worksheet::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 {
+    if (m_readOnly)
+        return;
+
     // forward the event to the items
     QGraphicsScene::contextMenuEvent(event);
 
@@ -1193,9 +1288,25 @@ void Worksheet::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
 void Worksheet::mousePressEvent(QGraphicsSceneMouseEvent* event)
 {
     QGraphicsScene::mousePressEvent(event);
+    /*
     if (event->button() == Qt::LeftButton && !focusItem() && lastEntry() &&
         event->scenePos().y() > lastEntry()->y() + lastEntry()->size().height())
         lastEntry()->focusEntry(WorksheetTextItem::BottomRight);
+    */
+    if (!m_readOnly)
+        updateEntryCursor(event);
+}
+
+void Worksheet::keyPressEvent(QKeyEvent *keyEvent)
+{
+    if (m_readOnly)
+        return;
+
+    // If we choose entry by entry cursor and press text button (not modifiers, for example, like Control)
+    if ((m_choosenCursorEntry || m_isCursorEntryAfterLastEntry) && !keyEvent->text().isEmpty())
+        addEntryFromEntryCursor();
+
+    QGraphicsScene::keyPressEvent(keyEvent);
 }
 
 void Worksheet::createActions(KActionCollection* collection)
@@ -1392,6 +1503,30 @@ WorksheetTextItem* Worksheet::lastFocusedTextItem()
 
 void Worksheet::updateFocusedTextItem(WorksheetTextItem* newItem)
 {
+    // No need update and emit signals about editing actions in readonly
+    // So support only copy action and reset selection
+    if (m_readOnly)
+    {
+        if (m_lastFocusedTextItem && m_lastFocusedTextItem != newItem)
+        {
+            disconnect(this, SIGNAL(copy()), m_lastFocusedTextItem, SLOT(copy()));
+            m_lastFocusedTextItem->clearSelection();
+        }
+
+        if (newItem && m_lastFocusedTextItem != newItem)
+        {
+            connect(this, SIGNAL(copy()), newItem, SLOT(copy()));
+            emit copyAvailable(newItem->isCopyAvailable());
+        }
+        else if (!newItem)
+        {
+            emit copyAvailable(false);
+        }
+
+        m_lastFocusedTextItem = newItem;
+        return;
+    }
+
     if (m_lastFocusedTextItem && m_lastFocusedTextItem != newItem) {
         disconnect(m_lastFocusedTextItem, SIGNAL(undoAvailable(bool)),
                    this, SIGNAL(undoAvailable(bool)));
@@ -1407,7 +1542,6 @@ void Worksheet::updateFocusedTextItem(WorksheetTextItem* newItem)
                    this, SIGNAL(pasteAvailable(bool)));
         disconnect(this, SIGNAL(cut()), m_lastFocusedTextItem, SLOT(cut()));
         disconnect(this, SIGNAL(copy()), m_lastFocusedTextItem, SLOT(copy()));
-        disconnect(this, SIGNAL(paste()), m_lastFocusedTextItem, SLOT(paste()));
 
         m_lastFocusedTextItem->clearSelection();
     }
@@ -1433,7 +1567,6 @@ void Worksheet::updateFocusedTextItem(WorksheetTextItem* newItem)
                 this, SIGNAL(pasteAvailable(bool)));
         connect(this, SIGNAL(cut()), newItem, SLOT(cut()));
         connect(this, SIGNAL(copy()), newItem, SLOT(copy()));
-        connect(this, SIGNAL(paste()), newItem, SLOT(paste()));
     } else if (!newItem) {
         emit undoAvailable(false);
         emit redoAvailable(false);
@@ -1444,6 +1577,19 @@ void Worksheet::updateFocusedTextItem(WorksheetTextItem* newItem)
     m_lastFocusedTextItem = newItem;
 }
 
+/*!
+ * handles the paste action triggered in cantor_part.
+ * Pastes into the last focused text item.
+ * In case the "new entry"-cursor is currently shown,
+ * a new entry is created first which the content will be pasted into.
+ */
+void Worksheet::paste() {
+    if (m_choosenCursorEntry || m_isCursorEntryAfterLastEntry)
+        addEntryFromEntryCursor();
+
+    m_lastFocusedTextItem->paste();
+}
+
 void Worksheet::setRichTextInformation(const RichTextInfo& info)
 {
     m_boldAction->setChecked(info.bold);
@@ -1466,19 +1612,9 @@ void Worksheet::setRichTextInformation(const RichTextInfo& info)
 
 void Worksheet::setAcceptRichText(bool b)
 {
-    foreach(QAction * action, m_richTextActionList) {
-        action->setEnabled(b);
-    }
-
-    /*
-    foreach(QWidget* widget, m_fontAction->createdWidgets()) {
-        widget->setEnabled(b);
-    }
-
-    foreach(QWidget* widget, m_fontSizeAction->createdWidgets()) {
-        widget->setEnabled(b);
-    }
-    */
+    if (!m_readOnly)
+        for(QAction * action : m_richTextActionList)
+            action->setEnabled(b);
 }
 
 WorksheetTextItem* Worksheet::currentTextItem()
@@ -1583,9 +1719,9 @@ bool Worksheet::isShortcut(const QKeySequence& sequence)
 
 void Worksheet::registerShortcut(QAction* action)
 {
-    foreach(QKeySequence shortcut, action->shortcuts()) {
+    for (auto& shortcut : action->shortcuts())
         m_shortcuts.insert(shortcut, action);
-    }
+
     connect(action, SIGNAL(changed()), this, SLOT(updateShortcut()));
 }
 
@@ -1597,13 +1733,12 @@ void Worksheet::updateShortcut()
 
     // delete the old shortcuts of this action
     QList<QKeySequence> shortcuts = m_shortcuts.keys(action);
-    foreach(QKeySequence shortcut, shortcuts) {
+    for (auto& shortcut : shortcuts)
         m_shortcuts.remove(shortcut);
-    }
+
     // add the new shortcuts
-    foreach(QKeySequence shortcut, action->shortcuts()) {
+    for (auto& shortcut : action->shortcuts())
         m_shortcuts.insert(shortcut, action);
-    }
 }
 
 void Worksheet::dragEnterEvent(QGraphicsSceneDragDropEvent* event)
@@ -1731,4 +1866,77 @@ void Worksheet::updateDragScrollTimer()
     m_dragScrollTimer->start();
 }
 
+void Worksheet::updateEntryCursor(QGraphicsSceneMouseEvent* event)
+{
+    // determine the worksheet entry near which the entry cursor will be shown
+    resetEntryCursor();
+    if (event->button() == Qt::LeftButton && !focusItem())
+    {
+        const qreal y = event->scenePos().y();
+        for (WorksheetEntry* entry = firstEntry(); entry; entry = entry->next())
+        {
+            if (entry == firstEntry() && y < entry->y() )
+            {
+                m_choosenCursorEntry = firstEntry();
+                break;
+            }
+            else if (entry->y() < y && (entry->next() && y < entry->next()->y()))
+            {
+                m_choosenCursorEntry = entry->next();
+                break;
+            }
+            else if (entry->y() < y && entry == lastEntry())
+            {
+                m_isCursorEntryAfterLastEntry = true;
+                break;
+            }
+        }
+    }
 
+    if (m_choosenCursorEntry || m_isCursorEntryAfterLastEntry)
+        drawEntryCursor();
+}
+
+void Worksheet::addEntryFromEntryCursor()
+{
+    qDebug() << "Add new entry from entry cursor";
+    if (m_isCursorEntryAfterLastEntry)
+        insertCommandEntry(lastEntry());
+    else
+        insertCommandEntryBefore(m_choosenCursorEntry);
+    resetEntryCursor();
+}
+
+void Worksheet::animateEntryCursor()
+{
+    if ((m_choosenCursorEntry || m_isCursorEntryAfterLastEntry) && m_entryCursorItem)
+        m_entryCursorItem->setVisible(!m_entryCursorItem->isVisible());
+}
+
+void Worksheet::resetEntryCursor()
+{
+    m_choosenCursorEntry = nullptr;
+    m_isCursorEntryAfterLastEntry = false;
+    m_entryCursorItem->hide();
+}
+
+void Worksheet::drawEntryCursor()
+{
+    if (m_entryCursorItem && (m_choosenCursorEntry || (m_isCursorEntryAfterLastEntry && lastEntry())))
+    {
+        qreal x;
+        qreal y;
+        if (m_isCursorEntryAfterLastEntry)
+        {
+            x = lastEntry()->x();
+            y = lastEntry()->y() + lastEntry()->size().height() - (EntryCursorWidth - 1);
+        }
+        else
+        {
+            x = m_choosenCursorEntry->x();
+            y = m_choosenCursorEntry->y();
+        }
+        m_entryCursorItem->setLine(x,y,x+EntryCursorLength,y);
+        m_entryCursorItem->show();
+    }
+}
