@@ -26,6 +26,7 @@
 #include <QDomElement>
 #include <QGraphicsLinearLayout>
 #include <QSyntaxHighlighter>
+#include <QGraphicsRectItem>
 
 #include <KZip>
 #include <QMenu>
@@ -62,6 +63,7 @@ class Worksheet : public QGraphicsScene
     Cantor::Session* session();
 
     bool isRunning();
+    bool isReadOnly();
     bool showExpressionIds();
     bool animationsEnabled();
 
@@ -71,8 +73,8 @@ class Worksheet : public QGraphicsScene
 
     WorksheetView* worksheetView();
 
-    void makeVisible(WorksheetEntry* entry);
-    void makeVisible(const WorksheetCursor& cursor);
+    void makeVisible(WorksheetEntry*);
+    void makeVisible(const WorksheetCursor&);
 
     void setModified();
 
@@ -115,27 +117,30 @@ class Worksheet : public QGraphicsScene
     WorksheetEntry* appendCommandEntry();
     void appendCommandEntry(const QString& text);
     WorksheetEntry* appendTextEntry();
+    WorksheetEntry* appendMarkdownEntry();
     WorksheetEntry* appendImageEntry();
     WorksheetEntry* appendPageBreakEntry();
     WorksheetEntry* appendLatexEntry();
     WorksheetEntry* insertCommandEntry(WorksheetEntry* current = nullptr);
     void insertCommandEntry(const QString& text);
     WorksheetEntry* insertTextEntry(WorksheetEntry* current = nullptr);
+    WorksheetEntry* insertMarkdownEntry(WorksheetEntry* current = nullptr);
     WorksheetEntry* insertImageEntry(WorksheetEntry* current = nullptr);
     WorksheetEntry* insertPageBreakEntry(WorksheetEntry* current = nullptr);
     WorksheetEntry* insertLatexEntry(WorksheetEntry* current = nullptr);
     WorksheetEntry* insertCommandEntryBefore(WorksheetEntry* current = nullptr);
     WorksheetEntry* insertTextEntryBefore(WorksheetEntry* current = nullptr);
+    WorksheetEntry* insertMarkdownEntryBefore(WorksheetEntry* current = nullptr);
     WorksheetEntry* insertImageEntryBefore(WorksheetEntry* current = nullptr);
     WorksheetEntry* insertPageBreakEntryBefore(WorksheetEntry* current = nullptr);
     WorksheetEntry* insertLatexEntryBefore(WorksheetEntry* current = nullptr);
 
     void updateLayout();
-    void updateEntrySize(WorksheetEntry* entry);
+    void updateEntrySize(WorksheetEntry*);
 
     void print(QPrinter*);
-
-    void focusEntry(WorksheetEntry * entry);
+    void paste();
+    void focusEntry(WorksheetEntry*);
 
     void evaluate();
     void evaluateCurrentEntry();
@@ -148,23 +153,23 @@ class Worksheet : public QGraphicsScene
     void highlightItem(WorksheetTextItem*);
     void rehighlight();
 
-    void enableHighlighting(bool highlight);
-    void enableCompletion(bool enable);
-    void enableExpressionNumbering(bool enable);
-    void enableAnimations(bool enable);
+    void enableHighlighting(bool);
+    void enableCompletion(bool);
+    void enableExpressionNumbering(bool);
+    void enableAnimations(bool);
 
-    QDomDocument toXML(KZip* archive=nullptr);
+    QDomDocument toXML(KZip* archive = nullptr);
 
     void save(const QString& filename);
-    void save(QIODevice* device);
+    void save(QIODevice*);
     QByteArray saveToByteArray();
     void savePlain(const QString& filename);
     void saveLatex(const QString& filename);
-    bool load(QIODevice* device);
+    bool load(QIODevice*);
     void load(QByteArray* data);
     bool load(const QString& filename);
 
-    void gotResult(Cantor::Expression* expr=nullptr);
+    void gotResult(Cantor::Expression* expr = nullptr);
 
     void removeCurrentEntry();
 
@@ -199,7 +204,7 @@ class Worksheet : public QGraphicsScene
 
   Q_SIGNALS:
     void modified();
-    void sessionChanged();
+    void loaded();
     void showHelp(const QString&);
     void updatePrompt();
     void undoAvailable(bool);
@@ -211,16 +216,17 @@ class Worksheet : public QGraphicsScene
     void pasteAvailable(bool);
     void cut();
     void copy();
-    void paste();
 
   protected:
-    void contextMenuEvent(QGraphicsSceneContextMenuEvent*) Q_DECL_OVERRIDE;
-    void mousePressEvent(QGraphicsSceneMouseEvent*) Q_DECL_OVERRIDE;
+    void contextMenuEvent(QGraphicsSceneContextMenuEvent*) override;
+    void mousePressEvent(QGraphicsSceneMouseEvent*) override;
 
-    void dragEnterEvent(QGraphicsSceneDragDropEvent*) Q_DECL_OVERRIDE;
-    void dragLeaveEvent(QGraphicsSceneDragDropEvent*) Q_DECL_OVERRIDE;
-    void dragMoveEvent(QGraphicsSceneDragDropEvent*) Q_DECL_OVERRIDE;
-    void dropEvent(QGraphicsSceneDragDropEvent*) Q_DECL_OVERRIDE;
+    void dragEnterEvent(QGraphicsSceneDragDropEvent*) override;
+    void dragLeaveEvent(QGraphicsSceneDragDropEvent*) override;
+    void dragMoveEvent(QGraphicsSceneDragDropEvent*) override;
+    void dropEvent(QGraphicsSceneDragDropEvent*) override;
+
+    void keyPressEvent(QKeyEvent*) override;
 
   private Q_SLOTS:
     void loginToSession();
@@ -231,22 +237,34 @@ class Worksheet : public QGraphicsScene
     WorksheetEntry* insertEntry(int type, WorksheetEntry* current = nullptr);
     WorksheetEntry* insertEntryBefore(int type, WorksheetEntry* current = nullptr);
 
+    void animateEntryCursor();
+
   private:
     WorksheetEntry* entryAt(qreal x, qreal y);
     WorksheetEntry* entryAt(QPointF p);
     WorksheetEntry* entryAt(int row);
+    void updateEntryCursor(QGraphicsSceneMouseEvent*);
+    void resetEntryCursor();
+    void addEntryFromEntryCursor();
+    void drawEntryCursor();
     int entryCount();
 
   private:
     static const double LeftMargin;
     static const double RightMargin;
     static const double TopMargin;
+    static const double EntryCursorLength;
+    static const double EntryCursorWidth;
     Cantor::Session *m_session;
     QSyntaxHighlighter* m_highlighter;
     EpsRenderer m_epsRenderer;
     WorksheetEntry* m_firstEntry;
     WorksheetEntry* m_lastEntry;
     WorksheetEntry* m_dragEntry;
+    WorksheetEntry* m_choosenCursorEntry;
+    bool m_isCursorEntryAfterLastEntry;
+    QTimer* m_cursorItemTimer;
+    QGraphicsLineItem* m_entryCursorItem;
     PlaceHolderEntry* m_placeholderEntry;
     WorksheetTextItem* m_lastFocusedTextItem;
     QTimer* m_dragScrollTimer;
@@ -275,6 +293,9 @@ class Worksheet : public QGraphicsScene
     bool m_loginDone;
     bool m_isPrinting;
     bool m_isLoadingFromFile;
+    bool m_readOnly;
+
+    QString m_backendName;
 };
 
 #endif // WORKSHEET_H
